/*
*  Name:      TSqlstate.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   SQLSTATE definitions
*  Date:      $Date: 2002/08/19 12:59:57 $
*  Revision:  $Revision: 1.1 $
*
*  Copyright (C) 2002  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Name-space for UESQL.
package org.uesqlc;


///  Class for holding the SQLSTATE related variables and values.
public class TSqlstate
{

  /**
  *  SQL state parameter (values are implementation defined and using it
  *  is deprecated).
  */
  public static int   SQLCODE = 0;

  /**
  *  SQL extended description (target dependent). This an extended message
  *  for including target dependent description of errors.
  */
  public static String   SQLMESSAGE = new String();

  ///  SQL state parameter (values in clause 22, "Status codes").
  public static String   SQLSTATE = new String ("00000");

  //
  //  SQLSTATE class and subclass values.
  //    Classes has been represented as a Java class, and subclasses have
  //    been represented as strings.
  //

  ///  Condition: ambiguous cursor name.
  public static class ambiguous_cursor_name
  {
    public static final String   _kyNO_SUBCLASS = new String ("3C000");
    public static final int      _kiNO_SUBCLASS = 0;
  }

  ///  Condition: cardinality violation.
  public static class cardinality_violation
  {
    public static final String   _kyNO_SUBCLASS = new String ("21000");
    public static final int      _kiNO_SUBCLASS = 1000;
  }
  
  ///  Condition: connection exception.
  public static class connection_exception
  {
    public static final String   _kyNO_SUBCLASS                                       = new String ("08000");
    public static final String   _kyCONNECTION_DOES_NOT_EXIST                         = new String ("08003");
    public static final String   _kyCONNECTION_FAILURE                                = new String ("08006");
    public static final String   _kyCONNECTION_NAME_IN_USE                            = new String ("08002");
    public static final String   _kySQLCLIENT_UNABLE_TO_ESTABLISH_SQLCONNECTION       = new String ("08001");
    public static final String   _kySQLSERVER_REJECTED_ESTABLISHMENT_OF_SQLCONNECTION = new String ("08004");
    public static final String   _kyTRANSACTION_RESOLUTION_UNKNOWN                    = new String ("08007");
    public static final int      _kiNO_SUBCLASS                                       = 2000;
    public static final int      _kiCONNECTION_DOES_NOT_EXIST                         = 2001;
    public static final int      _kiCONNECTION_FAILURE                                = 2002;
    public static final int      _kiCONNECTION_NAME_IN_USE                            = 2003;
    public static final int      _kiSQLCLIENT_UNABLE_TO_ESTABLISH_SQLCONNECTION       = 2004;
    public static final int      _kiSQLSERVER_REJECTED_ESTABLISHMENT_OF_SQLCONNECTION = 2005;
    public static final int      _kiTRANSACTION_RESOLUTION_UNKNOWN                    = 2006;
  }
  
  ///  Condition: data exception.
  public static class data_exception
  {
    public static final String   _kyNO_SUBCLASS                          = new String ("22000");
    public static final String   _kyCHARACTER_NOT_IN_REPERTOIRE          = new String ("22021");
    public static final String   _kyDATETIME_FIELD_OVERFLOW              = new String ("22008");
    public static final String   _kyDIVISION_BY_ZERO                     = new String ("22012");
    public static final String   _kyERROR_IN_ASSIGNMENT                  = new String ("22005");
    public static final String   _kyINDICATOR_OVERFLOW                   = new String ("22022");
    public static final String   _kyINTERVAL_FIELD_OVERFLOW              = new String ("22015");
    public static final String   _kyINVALID_CHARACTER_VALUE_FOR_CAST     = new String ("22018");
    public static final String   _kyINVALID_DATETIME_FORMAT              = new String ("22007");
    public static final String   _kyINVALID_ESCAPE_CHARACTER             = new String ("22019");
    public static final String   _kyINVALID_ESCAPE_SEQUENCE              = new String ("22025");
    public static final String   _kyINVALID_PARAMETER_VALUE              = new String ("22023");
    public static final String   _kyINVALID_TIME_ZONE_DISPLACEMENT_VALUE = new String ("22009");
    public static final String   _kyNULL_VALUE_NO_INDICATOR_PARAMETER    = new String ("22002");
    public static final String   _kyNUMERIC_VALUE_OUT_OF_RANGE           = new String ("22003");
    public static final String   _kySTRING_DATA_LENGTH_MISMATCH          = new String ("22026");
    public static final String   _kySTRING_DATA_RIGHT_TRUNCATION         = new String ("22001");
    public static final String   _kySUBSTRING_ERROR                      = new String ("22011");
    public static final String   _kyTRIM_ERROR                           = new String ("22027");
    public static final String   _kyUNTERMINATED_C_STRING                = new String ("22024");
    public static final int      _kiNO_SUBCLASS                          = 3000;
    public static final int      _kiCHARACTER_NOT_IN_REPERTOIRE          = 3001;
    public static final int      _kiDATETIME_FIELD_OVERFLOW              = 3002;
    public static final int      _kiDIVISION_BY_ZERO                     = 3003;
    public static final int      _kiERROR_IN_ASSIGNMENT                  = 3004;
    public static final int      _kiINDICATOR_OVERFLOW                   = 3005;
    public static final int      _kiINTERVAL_FIELD_OVERFLOW              = 3006;
    public static final int      _kiINVALID_CHARACTER_VALUE_FOR_CAST     = 3007;
    public static final int      _kiINVALID_DATETIME_FORMAT              = 3008;
    public static final int      _kiINVALID_ESCAPE_CHARACTER             = 3009;
    public static final int      _kiINVALID_ESCAPE_SEQUENCE              = 3010;
    public static final int      _kiINVALID_PARAMETER_VALUE              = 3011;
    public static final int      _kiINVALID_TIME_ZONE_DISPLACEMENT_VALUE = 3012;
    public static final int      _kiNULL_VALUE_NO_INDICATOR_PARAMETER    = 3013;
    public static final int      _kiNUMERIC_VALUE_OUT_OF_RANGE           = 3014;
    public static final int      _kiSTRING_DATA_LENGTH_MISMATCH          = 3015;
    public static final int      _kiSTRING_DATA_RIGHT_TRUNCATION         = 3016;
    public static final int      _kiSUBSTRING_ERROR                      = 3017;
    public static final int      _kiTRIM_ERROR                           = 3018;
    public static final int      _kiUNTERMINATED_C_STRING                = 3019;
  }
  
  ///  Condition: dependent privilege descriptors still exist.
  public static class dependent_privilege_descriptors_still_exist
  {
    public static final String   _kyNO_SUBCLASS = new String ("2B000");
    public static final int      _kiNO_SUBCLASS = 4000;
  }
  
  ///  Condition: dynamic SQL error.
  public static class dynamic_sql_error
  {
    public static final String   _kyNO_SUBCLASS                                        = new String ("07000");
    public static final String   _kyCURSOR_SPECIFICATION_CANNOT_BE_EXECUTED            = new String ("07003");
    public static final String   _kyINVALID_DESCRIPTOR_COUNT                           = new String ("07008");
    public static final String   _kyINVALID_DESCRIPTOR_INDEX                           = new String ("07009");
    public static final String   _kyPREPARED_STATEMENT_NOT_A_CURSOR_SPECIFICATION      = new String ("07005");
    public static final String   _kyRESTRICTED_DATA_TYPE_ATTRIBUTE_VIOLATION           = new String ("07006");
    public static final String   _kyUSING_CLAUSE_DOES_NOT_MATCH_DYNAMIC_PARAMETER_SPEC = new String ("07001");
    public static final String   _kyUSING_CLAUSE_DOES_NOT_MATCH_TARGET_SPEC            = new String ("07002");
    public static final String   _kyUSING_CLAUSE_REQUIRED_FOR_DYNAMIC_PARAMETERS       = new String ("07004");
    public static final String   _kyUSING_CLAUSE_REQUIRED_FOR_RESULT_FIELDS            = new String ("07007");
    public static final int      _kiNO_SUBCLASS                                        = 5000;
    public static final int      _kiCURSOR_SPECIFICATION_CANNOT_BE_EXECUTED            = 5001;
    public static final int      _kiINVALID_DESCRIPTOR_COUNT                           = 5002;
    public static final int      _kiINVALID_DESCRIPTOR_INDEX                           = 5003;
    public static final int      _kiPREPARED_STATEMENT_NOT_A_CURSOR_SPECIFICATION      = 5004;
    public static final int      _kiRESTRICTED_DATA_TYPE_ATTRIBUTE_VIOLATION           = 5005;
    public static final int      _kiUSING_CLAUSE_DOES_NOT_MATCH_DYNAMIC_PARAMETER_SPEC = 5006;
    public static final int      _kiUSING_CLAUSE_DOES_NOT_MATCH_TARGET_SPEC            = 5007;
    public static final int      _kiUSING_CLAUSE_REQUIRED_FOR_DYNAMIC_PARAMETERS       = 5008;
    public static final int      _kiUSING_CLAUSE_REQUIRED_FOR_RESULT_FIELDS            = 5009;
  }
  
  ///  Condition: feature not supported.
  public static class feature_not_supported
  {
    public static final String   _kyNO_SUBCLASS                       = new String ("0A000");
    public static final String   _kyMULTIPLE_ENVIRONMENT_TRANSACTIONS = new String ("0A001");
    public static final int      _kiNO_SUBCLASS                       = 6000;
    public static final int      _kiMULTIPLE_ENVIRONMENT_TRANSACTIONS = 6001;
  }
  
  ///  Condition: integrity constraint violation.
  public static class integrity_constraint_violation
  {
    public static final String   _kyNO_SUBCLASS = new String ("23000");
    public static final int      _kiNO_SUBCLASS = 7000;
  }
  
  ///  Condition: invalid authorization specification.
  public static class invalid_authorization_specification
  {
    public static final String   _kyNO_SUBCLASS = new String ("28000");
    public static final int      _kiNO_SUBCLASS = 8000;
  }
  
  ///  Condition: invalid catalog name.
  public static class invalid_catalog_name
  {
    public static final String   _kyNO_SUBCLASS = new String ("3D000");
    public static final int      _kiNO_SUBCLASS = 9000;
  }
  
  ///  Condition: invalid character set name.
  public static class invalid_character_set_name
  {
    public static final String   _kyNO_SUBCLASS = new String ("2C000");
    public static final int      _kiNO_SUBCLASS = 10000;
  }
  
  ///  Condition: invalid condition number.
  public static class invalid_condition_number
  {
    public static final String   _kyNO_SUBCLASS = new String ("35000");
    public static final int      _kiNO_SUBCLASS = 11000;
  }
  
  ///  Condition: invalid connection name.
  public static class invalid_connection_name
  {
    public static final String   _kyNO_SUBCLASS = new String ("2E000");
    public static final int      _kiNO_SUBCLASS = 12000;
  }
  
  ///  Condition: invalid cursor name.
  public static class invalid_cursor_name
  {
    public static final String   _kyNO_SUBCLASS = new String ("34000");
    public static final int      _kiNO_SUBCLASS = 13000;
  }
  
  ///  Condition: invalid cursor state.
  public static class invalid_cursor_state
  {
    public static final String   _kyNO_SUBCLASS = new String ("24000");
    public static final int      _kiNO_SUBCLASS = 14000;
  }
  
  ///  Condition: invalid schema name.
  public static class invalid_schema_name
  {
    public static final String   _kyNO_SUBCLASS = new String ("3F000");
    public static final int      _kiNO_SUBCLASS = 15000;
  }
  
  ///  Condition: invalid SQL descriptor name.
  public static class invalid_sql_descriptor_name
  {
    public static final String   _kyNO_SUBCLASS = new String ("33000");
    public static final int      _kiNO_SUBCLASS = 16000;
  }
  
  ///  Condition: invalid SQL statement name.
  public static class invalid_sql_statement_name
  {
    public static final String   _kyNO_SUBCLASS = new String ("26000");
    public static final int      _kiNO_SUBCLASS = 17000;
  }
  
  ///  Condition: invalid transaction state.
  public static class invalid_transaction_state
  {
    public static final String   _kyNO_SUBCLASS = new String ("25000");
    public static final int      _kiNO_SUBCLASS = 18000;
  }
  
  ///  Condition: invalid transaction termination.
  public static class invalid_transaction_termination
  {
    public static final String   _kyNO_SUBCLASS = new String ("2D000");
    public static final int      _kiNO_SUBCLASS = 19000;
  }
  
  ///  Condition: no data.
  public static class no_data
  {
    public static final String   _kyNO_SUBCLASS = new String ("02000");
    public static final int      _kiNO_SUBCLASS = 20000;
  }
  
  ///  Condition: remote database access.
  public static class remote_database_access
  {
    public static final String   _kyNO_SUBCLASS = new String ("HZ000");
    public static final int      _kiNO_SUBCLASS = 21000;
  }
  
  ///  Condition: successful completion.
  public static class successful_completion
  {
    public static final String   _kyNO_SUBCLASS = new String ("00000");
    public static final int      _kiNO_SUBCLASS = 22000;
  }
  
  ///  Condition: syntax error or access rule violation.
  public static class syntax_error_or_access_rule_violation
  {
    public static final String   _kyNO_SUBCLASS = new String ("42000");
    public static final int      _kiNO_SUBCLASS = 23000;
  }
  
  ///  Condition: syntax error or access rule violation in direct statement.
  public static class syntax_error_or_access_rule_violation_in_direct_statement
  {
    public static final String   _kyNO_SUBCLASS = new String ("2A000");
    public static final int      _kiNO_SUBCLASS = 24000;
  }
  
  ///  Condition: syntax error or access rule violation in dynamic statement.
  public static class syntax_error_or_access_rule_violation_in_dynamic_statement
  {
    public static final String   _kyNO_SUBCLASS = new String ("37000");
    public static final int      _kiNO_SUBCLASS = 25000;
  }
  
  ///  Condition: transaction rollback.
  public static class transaction_rollback
  {
    public static final String   _kyNO_SUBCLASS                    = new String ("40000");
    public static final String   _kyINTEGRITY_CONSTRAINT_VIOLATION = new String ("40002");
    public static final String   _kySERIALIZATION_FAILURE          = new String ("40001");
    public static final String   _kySTATEMENT_COMPLETION_UNKNOWN   = new String ("40003");
    public static final int      _kiNO_SUBCLASS                    = 26000;
    public static final int      _kiINTEGRITY_CONSTRAINT_VIOLATION = 26001;
    public static final int      _kiSERIALIZATION_FAILURE          = 26002;
    public static final int      _kiSTATEMENT_COMPLETION_UNKNOWN   = 26003;
  }
  
  ///  Condition: triggered data change violation.
  public static class triggered_data_change_violation
  {
    public static final String   _kyNO_SUBCLASS = new String ("27000");
    public static final int      _kiNO_SUBCLASS = 27000;
  }
  
  ///  Condition: warning.
  public static class warning
  {
    public static final String   _kyNO_SUBCLASS                                      = new String ("01000");
    public static final String   _kyCURSOR_OPERATION_CONFLICT                        = new String ("01001");
    public static final String   _kyDISCONNECT_ERROR                                 = new String ("01002");
    public static final String   _kyIMPLICIT_ZERO_BIT_PADDING                        = new String ("01008");
    public static final String   _kyINSUFFICIENT_ITEM_DESCRIPTOR_AREAS               = new String ("01005");
    public static final String   _kyNULL_VALUE_ELIMINATED_IN_SET_FUNCTION            = new String ("01003");
    public static final String   _kyPRIVILEGE_NOT_GRANTED                            = new String ("01007");
    public static final String   _kyPRIVILEGE_NOT_REVOKED                            = new String ("01006");
    public static final String   _kyQUERY_EXPRESSION_TOO_LONG_FOR_INFORMATION_SCHEMA = new String ("0100A");
    public static final String   _kySEARCH_CONDITION_TOO_LONG_FOR_INFORMATION_SCHEMA = new String ("01009");
    public static final String   _kySTRING_DATA_RIGHT_TRUNCATION                     = new String ("01004");
    public static final int      _kiNO_SUBCLASS                                      = 28000;
    public static final int      _kiCURSOR_OPERATION_CONFLICT                        = 28001;
    public static final int      _kiDISCONNECT_ERROR                                 = 28002;
    public static final int      _kiIMPLICIT_ZERO_BIT_PADDING                        = 28003;
    public static final int      _kiINSUFFICIENT_ITEM_DESCRIPTOR_AREAS               = 28004;
    public static final int      _kiNULL_VALUE_ELIMINATED_IN_SET_FUNCTION            = 28005;
    public static final int      _kiPRIVILEGE_NOT_GRANTED                            = 28006;
    public static final int      _kiPRIVILEGE_NOT_REVOKED                            = 28007;
    public static final int      _kiQUERY_EXPRESSION_TOO_LONG_FOR_INFORMATION_SCHEMA = 28008;
    public static final int      _kiSEARCH_CONDITION_TOO_LONG_FOR_INFORMATION_SCHEMA = 28009;
    public static final int      _kiSTRING_DATA_RIGHT_TRUNCATION                     = 28010;
  }
  
  ///  Condition: with check option violation.
  public static class with_check_option_violation
  {
    public static final String   _kyNO_SUBCLASS = new String ("44000");
    public static final int      _kiNO_SUBCLASS = 29000;
  }
  
}  // class TSqlstate
