/*
*  Name:      TSqlstateDescriptor.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Class for handling the SQLSTATE status parameter.
*  Date:      $Date: 2002/08/19 12:59:57 $
*  Revision:  $Revision: 1.1 $
*
*  Copyright (C) 2002  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/


///  Name-space for UESQL.
package org.uesqlc;

import java.util.Hashtable;
import org.uesqlc.TSqlstate;


///  Handler for SQLSTATE status parameter.
class TSqlstateDescriptor
{

  /**
  *  Map with SQLSTATE values (strings) to integer translations.
  *  So the integer values are suitable for a \b switch statement.
  */
  private static Hashtable   _tValueToIntMap;

  //
  //  L O C A L   F U N C T I O N S
  //

  ///  Initializes the SQLSTATE to integer map.
  static
  {

    _tValueToIntMap.put (TSqlstate.ambiguous_cursor_name._kyNO_SUBCLASS,                                      new Integer (TSqlstate.ambiguous_cursor_name._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.cardinality_violation._kyNO_SUBCLASS,                                      new Integer (TSqlstate.cardinality_violation._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.connection_exception._kyNO_SUBCLASS,                                       new Integer (TSqlstate.connection_exception._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.connection_exception._kyCONNECTION_DOES_NOT_EXIST,                         new Integer (TSqlstate.connection_exception._kiCONNECTION_DOES_NOT_EXIST));
    _tValueToIntMap.put (TSqlstate.connection_exception._kyCONNECTION_FAILURE,                                new Integer (TSqlstate.connection_exception._kiCONNECTION_FAILURE));
    _tValueToIntMap.put (TSqlstate.connection_exception._kyCONNECTION_NAME_IN_USE,                            new Integer (TSqlstate.connection_exception._kiCONNECTION_NAME_IN_USE));
    _tValueToIntMap.put (TSqlstate.connection_exception._kySQLCLIENT_UNABLE_TO_ESTABLISH_SQLCONNECTION,       new Integer (TSqlstate.connection_exception._kiSQLCLIENT_UNABLE_TO_ESTABLISH_SQLCONNECTION));
    _tValueToIntMap.put (TSqlstate.connection_exception._kySQLSERVER_REJECTED_ESTABLISHMENT_OF_SQLCONNECTION, new Integer (TSqlstate.connection_exception._kiSQLSERVER_REJECTED_ESTABLISHMENT_OF_SQLCONNECTION));
    _tValueToIntMap.put (TSqlstate.connection_exception._kyTRANSACTION_RESOLUTION_UNKNOWN,                    new Integer (TSqlstate.connection_exception._kiTRANSACTION_RESOLUTION_UNKNOWN));
    _tValueToIntMap.put (TSqlstate.data_exception._kyNO_SUBCLASS,                                             new Integer (TSqlstate.data_exception._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.data_exception._kyCHARACTER_NOT_IN_REPERTOIRE,                             new Integer (TSqlstate.data_exception._kiCHARACTER_NOT_IN_REPERTOIRE));
    _tValueToIntMap.put (TSqlstate.data_exception._kyDATETIME_FIELD_OVERFLOW,                                 new Integer (TSqlstate.data_exception._kiDATETIME_FIELD_OVERFLOW));
    _tValueToIntMap.put (TSqlstate.data_exception._kyDIVISION_BY_ZERO,                                        new Integer (TSqlstate.data_exception._kiDIVISION_BY_ZERO));
    _tValueToIntMap.put (TSqlstate.data_exception._kyERROR_IN_ASSIGNMENT,                                     new Integer (TSqlstate.data_exception._kiERROR_IN_ASSIGNMENT));
    _tValueToIntMap.put (TSqlstate.data_exception._kyINDICATOR_OVERFLOW,                                      new Integer (TSqlstate.data_exception._kiINDICATOR_OVERFLOW));
    _tValueToIntMap.put (TSqlstate.data_exception._kyINTERVAL_FIELD_OVERFLOW,                                 new Integer (TSqlstate.data_exception._kiINTERVAL_FIELD_OVERFLOW));
    _tValueToIntMap.put (TSqlstate.data_exception._kyINVALID_CHARACTER_VALUE_FOR_CAST,                        new Integer (TSqlstate.data_exception._kiINVALID_CHARACTER_VALUE_FOR_CAST));
    _tValueToIntMap.put (TSqlstate.data_exception._kyINVALID_DATETIME_FORMAT,                                 new Integer (TSqlstate.data_exception._kiINVALID_DATETIME_FORMAT));
    _tValueToIntMap.put (TSqlstate.data_exception._kyINVALID_ESCAPE_CHARACTER,                                new Integer (TSqlstate.data_exception._kiINVALID_ESCAPE_CHARACTER));
    _tValueToIntMap.put (TSqlstate.data_exception._kyINVALID_ESCAPE_SEQUENCE,                                 new Integer (TSqlstate.data_exception._kiINVALID_ESCAPE_SEQUENCE));
    _tValueToIntMap.put (TSqlstate.data_exception._kyINVALID_PARAMETER_VALUE,                                 new Integer (TSqlstate.data_exception._kiINVALID_PARAMETER_VALUE));
    _tValueToIntMap.put (TSqlstate.data_exception._kyINVALID_TIME_ZONE_DISPLACEMENT_VALUE,                    new Integer (TSqlstate.data_exception._kiINVALID_TIME_ZONE_DISPLACEMENT_VALUE));
    _tValueToIntMap.put (TSqlstate.data_exception._kyNULL_VALUE_NO_INDICATOR_PARAMETER,                       new Integer (TSqlstate.data_exception._kiNULL_VALUE_NO_INDICATOR_PARAMETER));
    _tValueToIntMap.put (TSqlstate.data_exception._kyNUMERIC_VALUE_OUT_OF_RANGE,                              new Integer (TSqlstate.data_exception._kiNUMERIC_VALUE_OUT_OF_RANGE));
    _tValueToIntMap.put (TSqlstate.data_exception._kySTRING_DATA_LENGTH_MISMATCH,                             new Integer (TSqlstate.data_exception._kiSTRING_DATA_LENGTH_MISMATCH));
    _tValueToIntMap.put (TSqlstate.data_exception._kySTRING_DATA_RIGHT_TRUNCATION,                            new Integer (TSqlstate.data_exception._kiSTRING_DATA_RIGHT_TRUNCATION));
    _tValueToIntMap.put (TSqlstate.data_exception._kySUBSTRING_ERROR,                                         new Integer (TSqlstate.data_exception._kiSUBSTRING_ERROR));
    _tValueToIntMap.put (TSqlstate.data_exception._kyTRIM_ERROR,                                              new Integer (TSqlstate.data_exception._kiTRIM_ERROR));
    _tValueToIntMap.put (TSqlstate.data_exception._kyUNTERMINATED_C_STRING,                                   new Integer (TSqlstate.data_exception._kiUNTERMINATED_C_STRING));
    _tValueToIntMap.put (TSqlstate.dependent_privilege_descriptors_still_exist._kyNO_SUBCLASS,                new Integer (TSqlstate.dependent_privilege_descriptors_still_exist._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.dynamic_sql_error._kyNO_SUBCLASS,                                          new Integer (TSqlstate.dynamic_sql_error._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.dynamic_sql_error._kyCURSOR_SPECIFICATION_CANNOT_BE_EXECUTED,              new Integer (TSqlstate.dynamic_sql_error._kiCURSOR_SPECIFICATION_CANNOT_BE_EXECUTED));
    _tValueToIntMap.put (TSqlstate.dynamic_sql_error._kyINVALID_DESCRIPTOR_COUNT,                             new Integer (TSqlstate.dynamic_sql_error._kiINVALID_DESCRIPTOR_COUNT));
    _tValueToIntMap.put (TSqlstate.dynamic_sql_error._kyINVALID_DESCRIPTOR_INDEX,                             new Integer (TSqlstate.dynamic_sql_error._kiINVALID_DESCRIPTOR_INDEX));
    _tValueToIntMap.put (TSqlstate.dynamic_sql_error._kyPREPARED_STATEMENT_NOT_A_CURSOR_SPECIFICATION,        new Integer (TSqlstate.dynamic_sql_error._kiPREPARED_STATEMENT_NOT_A_CURSOR_SPECIFICATION));
    _tValueToIntMap.put (TSqlstate.dynamic_sql_error._kyRESTRICTED_DATA_TYPE_ATTRIBUTE_VIOLATION,             new Integer (TSqlstate.dynamic_sql_error._kiRESTRICTED_DATA_TYPE_ATTRIBUTE_VIOLATION));
    _tValueToIntMap.put (TSqlstate.dynamic_sql_error._kyUSING_CLAUSE_DOES_NOT_MATCH_DYNAMIC_PARAMETER_SPEC,   new Integer (TSqlstate.dynamic_sql_error._kiUSING_CLAUSE_DOES_NOT_MATCH_DYNAMIC_PARAMETER_SPEC));
    _tValueToIntMap.put (TSqlstate.dynamic_sql_error._kyUSING_CLAUSE_DOES_NOT_MATCH_TARGET_SPEC,              new Integer (TSqlstate.dynamic_sql_error._kiUSING_CLAUSE_DOES_NOT_MATCH_TARGET_SPEC));
    _tValueToIntMap.put (TSqlstate.dynamic_sql_error._kyUSING_CLAUSE_REQUIRED_FOR_DYNAMIC_PARAMETERS,         new Integer (TSqlstate.dynamic_sql_error._kiUSING_CLAUSE_REQUIRED_FOR_DYNAMIC_PARAMETERS));
    _tValueToIntMap.put (TSqlstate.dynamic_sql_error._kyUSING_CLAUSE_REQUIRED_FOR_RESULT_FIELDS,              new Integer (TSqlstate.dynamic_sql_error._kiUSING_CLAUSE_REQUIRED_FOR_RESULT_FIELDS));
    _tValueToIntMap.put (TSqlstate.feature_not_supported._kyNO_SUBCLASS,                                      new Integer (TSqlstate.feature_not_supported._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.feature_not_supported._kyMULTIPLE_ENVIRONMENT_TRANSACTIONS,                new Integer (TSqlstate.feature_not_supported._kiMULTIPLE_ENVIRONMENT_TRANSACTIONS));
    _tValueToIntMap.put (TSqlstate.integrity_constraint_violation._kyNO_SUBCLASS,                             new Integer (TSqlstate.integrity_constraint_violation._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.invalid_authorization_specification._kyNO_SUBCLASS,                        new Integer (TSqlstate.invalid_authorization_specification._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.invalid_catalog_name._kyNO_SUBCLASS,                                       new Integer (TSqlstate.invalid_catalog_name._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.invalid_character_set_name._kyNO_SUBCLASS,                                 new Integer (TSqlstate.invalid_character_set_name._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.invalid_condition_number._kyNO_SUBCLASS,                                   new Integer (TSqlstate.invalid_condition_number._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.invalid_connection_name._kyNO_SUBCLASS,                                    new Integer (TSqlstate.invalid_connection_name._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.invalid_cursor_name._kyNO_SUBCLASS,                                        new Integer (TSqlstate.invalid_cursor_name._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.invalid_cursor_state._kyNO_SUBCLASS,                                       new Integer (TSqlstate.invalid_cursor_state._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.invalid_schema_name._kyNO_SUBCLASS,                                        new Integer (TSqlstate.invalid_schema_name._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.invalid_sql_descriptor_name._kyNO_SUBCLASS,                                new Integer (TSqlstate.invalid_sql_descriptor_name._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.invalid_sql_statement_name._kyNO_SUBCLASS,                                 new Integer (TSqlstate.invalid_sql_statement_name._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.invalid_transaction_state._kyNO_SUBCLASS,                                  new Integer (TSqlstate.invalid_transaction_state._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.invalid_transaction_termination._kyNO_SUBCLASS,                            new Integer (TSqlstate.invalid_transaction_termination._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.no_data._kyNO_SUBCLASS,                                                    new Integer (TSqlstate.no_data._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.remote_database_access._kyNO_SUBCLASS,                                     new Integer (TSqlstate.remote_database_access._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.successful_completion._kyNO_SUBCLASS,                                      new Integer (TSqlstate.successful_completion._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.syntax_error_or_access_rule_violation._kyNO_SUBCLASS,                      new Integer (TSqlstate.syntax_error_or_access_rule_violation._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.syntax_error_or_access_rule_violation_in_direct_statement._kyNO_SUBCLASS,  new Integer (TSqlstate.syntax_error_or_access_rule_violation_in_direct_statement._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.syntax_error_or_access_rule_violation_in_dynamic_statement._kyNO_SUBCLASS, new Integer (TSqlstate.syntax_error_or_access_rule_violation_in_dynamic_statement._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.transaction_rollback._kyNO_SUBCLASS,                                       new Integer (TSqlstate.transaction_rollback._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.transaction_rollback._kyINTEGRITY_CONSTRAINT_VIOLATION,                    new Integer (TSqlstate.transaction_rollback._kiINTEGRITY_CONSTRAINT_VIOLATION));
    _tValueToIntMap.put (TSqlstate.transaction_rollback._kySERIALIZATION_FAILURE,                             new Integer (TSqlstate.transaction_rollback._kiSERIALIZATION_FAILURE));
    _tValueToIntMap.put (TSqlstate.transaction_rollback._kySTATEMENT_COMPLETION_UNKNOWN,                      new Integer (TSqlstate.transaction_rollback._kiSTATEMENT_COMPLETION_UNKNOWN));
    _tValueToIntMap.put (TSqlstate.triggered_data_change_violation._kyNO_SUBCLASS,                            new Integer (TSqlstate.triggered_data_change_violation._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.warning._kyNO_SUBCLASS,                                                    new Integer (TSqlstate.warning._kiNO_SUBCLASS));
    _tValueToIntMap.put (TSqlstate.warning._kyCURSOR_OPERATION_CONFLICT,                                      new Integer (TSqlstate.warning._kiCURSOR_OPERATION_CONFLICT));
    _tValueToIntMap.put (TSqlstate.warning._kyDISCONNECT_ERROR,                                               new Integer (TSqlstate.warning._kiDISCONNECT_ERROR));
    _tValueToIntMap.put (TSqlstate.warning._kyIMPLICIT_ZERO_BIT_PADDING,                                      new Integer (TSqlstate.warning._kiIMPLICIT_ZERO_BIT_PADDING));
    _tValueToIntMap.put (TSqlstate.warning._kyINSUFFICIENT_ITEM_DESCRIPTOR_AREAS,                             new Integer (TSqlstate.warning._kiINSUFFICIENT_ITEM_DESCRIPTOR_AREAS));
    _tValueToIntMap.put (TSqlstate.warning._kyNULL_VALUE_ELIMINATED_IN_SET_FUNCTION,                          new Integer (TSqlstate.warning._kiNULL_VALUE_ELIMINATED_IN_SET_FUNCTION));
    _tValueToIntMap.put (TSqlstate.warning._kyPRIVILEGE_NOT_GRANTED,                                          new Integer (TSqlstate.warning._kiPRIVILEGE_NOT_GRANTED));
    _tValueToIntMap.put (TSqlstate.warning._kyPRIVILEGE_NOT_REVOKED,                                          new Integer (TSqlstate.warning._kiPRIVILEGE_NOT_REVOKED));
    _tValueToIntMap.put (TSqlstate.warning._kyQUERY_EXPRESSION_TOO_LONG_FOR_INFORMATION_SCHEMA,               new Integer (TSqlstate.warning._kiQUERY_EXPRESSION_TOO_LONG_FOR_INFORMATION_SCHEMA));
    _tValueToIntMap.put (TSqlstate.warning._kySEARCH_CONDITION_TOO_LONG_FOR_INFORMATION_SCHEMA,               new Integer (TSqlstate.warning._kiSEARCH_CONDITION_TOO_LONG_FOR_INFORMATION_SCHEMA));
    _tValueToIntMap.put (TSqlstate.warning._kySTRING_DATA_RIGHT_TRUNCATION,                                   new Integer (TSqlstate.warning._kiSTRING_DATA_RIGHT_TRUNCATION));
    _tValueToIntMap.put (TSqlstate.with_check_option_violation._kyNO_SUBCLASS,                                new Integer (TSqlstate.with_check_option_violation._kiNO_SUBCLASS));

  }  // static


  //
  //  S E L E C T O R S
  //

  /**
  *  Returns true if the query is empty.
  *  @return True if query is empty.
  */
  public boolean queryIsEmpty()
  {
    return ( getUniqueValue() == TSqlstate.no_data._kiNO_SUBCLASS );
  }

  /**
  *  Returns true if the query has only a single row.
  *  @return True if query has a single row.
  */
  public boolean queryHasSingleRow()
  {
    //
    //  Check if query has generated only *one* row.
    //
    return ( ( getUniqueValue() != TSqlstate.cardinality_violation._kiNO_SUBCLASS ) &&
             ( getUniqueValue() != TSqlstate.no_data._kiNO_SUBCLASS )                );
  }

  /**
  *  Finds the integer value for the current SQLSTATE.
  *  @return The integer value for the current SQLSTATE.
  */
  public int getUniqueValue()
  {
    return ((Integer) _tValueToIntMap.get (TSqlstate.SQLSTATE)).intValue();
  }

  /**
  *  Finds the integer value for the SQLSTATE in \a ySQLSTATE.
  *  @param ySQLSTATE SQLSTATE value.
  *  @return The integer value for the SQLSTATE in \a ySQLSTATE.
  */
  public int getUniqueValue (final String ySQLSTATE)
  {
    return ((Integer) _tValueToIntMap.get (ySQLSTATE)).intValue();
  }

  /**
  *  Gets SQLSTATE class.
  *  @return Class description.
  */
  public final String getClassValue()
  {
    return getClassValue (TSqlstate.SQLSTATE);
  }

  /**
  *  Gets SQLSTATE class for \a ySQLSTATE.
  *  @return Class description.
  */
  public final String getClassValue (final String kySQLSTATE)
  {

    String   yClass = new String();

    switch (getUniqueValue (kySQLSTATE))
    {
      case TSqlstate.ambiguous_cursor_name._kiNO_SUBCLASS:
      {
        yClass = "ambiguous cursor name";
        break;
      }
      case TSqlstate.cardinality_violation._kiNO_SUBCLASS:
      {
        yClass = "cardinality violation";
        break;
      }
      case TSqlstate.connection_exception._kiNO_SUBCLASS:
      case TSqlstate.connection_exception._kiCONNECTION_DOES_NOT_EXIST:
      case TSqlstate.connection_exception._kiCONNECTION_FAILURE:
      case TSqlstate.connection_exception._kiCONNECTION_NAME_IN_USE:
      case TSqlstate.connection_exception._kiSQLCLIENT_UNABLE_TO_ESTABLISH_SQLCONNECTION:
      case TSqlstate.connection_exception._kiSQLSERVER_REJECTED_ESTABLISHMENT_OF_SQLCONNECTION:
      case TSqlstate.connection_exception._kiTRANSACTION_RESOLUTION_UNKNOWN:
      {
        yClass = "connection exception";
        break;
      }
      case TSqlstate.data_exception._kiNO_SUBCLASS:
      case TSqlstate.data_exception._kiCHARACTER_NOT_IN_REPERTOIRE:
      case TSqlstate.data_exception._kiDATETIME_FIELD_OVERFLOW:
      case TSqlstate.data_exception._kiDIVISION_BY_ZERO:
      case TSqlstate.data_exception._kiERROR_IN_ASSIGNMENT:
      case TSqlstate.data_exception._kiINDICATOR_OVERFLOW:
      case TSqlstate.data_exception._kiINTERVAL_FIELD_OVERFLOW:
      case TSqlstate.data_exception._kiINVALID_CHARACTER_VALUE_FOR_CAST:
      case TSqlstate.data_exception._kiINVALID_DATETIME_FORMAT:
      case TSqlstate.data_exception._kiINVALID_ESCAPE_CHARACTER:
      case TSqlstate.data_exception._kiINVALID_ESCAPE_SEQUENCE:
      case TSqlstate.data_exception._kiINVALID_PARAMETER_VALUE:
      case TSqlstate.data_exception._kiINVALID_TIME_ZONE_DISPLACEMENT_VALUE:
      case TSqlstate.data_exception._kiNULL_VALUE_NO_INDICATOR_PARAMETER:
      case TSqlstate.data_exception._kiNUMERIC_VALUE_OUT_OF_RANGE:
      case TSqlstate.data_exception._kiSTRING_DATA_LENGTH_MISMATCH:
      case TSqlstate.data_exception._kiSTRING_DATA_RIGHT_TRUNCATION:
      case TSqlstate.data_exception._kiSUBSTRING_ERROR:
      case TSqlstate.data_exception._kiTRIM_ERROR:
      case TSqlstate.data_exception._kiUNTERMINATED_C_STRING:
      {
        yClass = "data exception";
        break;
      }
      case TSqlstate.dependent_privilege_descriptors_still_exist._kiNO_SUBCLASS:
      {
        yClass = "dependent privilege descriptors still exist";
        break;
      }
      case TSqlstate.dynamic_sql_error._kiNO_SUBCLASS:
      case TSqlstate.dynamic_sql_error._kiCURSOR_SPECIFICATION_CANNOT_BE_EXECUTED:
      case TSqlstate.dynamic_sql_error._kiINVALID_DESCRIPTOR_COUNT:
      case TSqlstate.dynamic_sql_error._kiINVALID_DESCRIPTOR_INDEX:
      case TSqlstate.dynamic_sql_error._kiPREPARED_STATEMENT_NOT_A_CURSOR_SPECIFICATION:
      case TSqlstate.dynamic_sql_error._kiRESTRICTED_DATA_TYPE_ATTRIBUTE_VIOLATION:
      case TSqlstate.dynamic_sql_error._kiUSING_CLAUSE_DOES_NOT_MATCH_DYNAMIC_PARAMETER_SPEC:
      case TSqlstate.dynamic_sql_error._kiUSING_CLAUSE_DOES_NOT_MATCH_TARGET_SPEC:
      case TSqlstate.dynamic_sql_error._kiUSING_CLAUSE_REQUIRED_FOR_DYNAMIC_PARAMETERS:
      case TSqlstate.dynamic_sql_error._kiUSING_CLAUSE_REQUIRED_FOR_RESULT_FIELDS:
      {
        yClass = "dynamic SQL error";
        break;
      }
      case TSqlstate.feature_not_supported._kiNO_SUBCLASS:
      case TSqlstate.feature_not_supported._kiMULTIPLE_ENVIRONMENT_TRANSACTIONS:
      {
        yClass = "feature not supported";
        break;
      }
      case TSqlstate.integrity_constraint_violation._kiNO_SUBCLASS:
      {
        yClass = "integrity constraint violation";
        break;
      }
      case TSqlstate.invalid_authorization_specification._kiNO_SUBCLASS:
      {
        yClass = "invalid authorization specification";
        break;
      }
      case TSqlstate.invalid_catalog_name._kiNO_SUBCLASS:
      {
        yClass = "invalid catalog name";
        break;
      }
      case TSqlstate.invalid_character_set_name._kiNO_SUBCLASS:
      {
        yClass = "invalid character set name";
        break;
      }
      case TSqlstate.invalid_condition_number._kiNO_SUBCLASS:
      {
        yClass = "invalid condition number";
        break;
      }
      case TSqlstate.invalid_connection_name._kiNO_SUBCLASS:
      {
        yClass = "invalid connection name";
        break;
      }
      case TSqlstate.invalid_cursor_name._kiNO_SUBCLASS:
      {
        yClass = "invalid cursor name";
        break;
      }
      case TSqlstate.invalid_cursor_state._kiNO_SUBCLASS:
      {
        yClass = "invalid cursor state";
        break;
      }
      case TSqlstate.invalid_schema_name._kiNO_SUBCLASS:
      {
        yClass = "invalid schema name";
        break;
      }
      case TSqlstate.invalid_sql_descriptor_name._kiNO_SUBCLASS:
      {
        yClass = "invalid SQL descriptor name";
        break;
      }
      case TSqlstate.invalid_sql_statement_name._kiNO_SUBCLASS:
      {
        yClass = "invalid SQL statement name";
        break;
      }
      case TSqlstate.invalid_transaction_state._kiNO_SUBCLASS:
      {
        yClass = "invalid transaction state";
        break;
      }
      case TSqlstate.invalid_transaction_termination._kiNO_SUBCLASS:
      {
        yClass = "invalid transaction termination";
        break;
      }
      case TSqlstate.no_data._kiNO_SUBCLASS:
      {
        yClass = "no data";
        break;
      }
      case TSqlstate.remote_database_access._kiNO_SUBCLASS:
      {
        yClass = "remote database access";
        break;
      }
      case TSqlstate.successful_completion._kiNO_SUBCLASS:
      {
        yClass = "successful completion";
        break;
      }
      case TSqlstate.syntax_error_or_access_rule_violation._kiNO_SUBCLASS:
      {
        yClass = "syntax error or access rule violation";
        break;
      }
      case TSqlstate.syntax_error_or_access_rule_violation_in_direct_statement._kiNO_SUBCLASS:
      {
        yClass = "syntax error or access rule violation in direct statement";
        break;
      }
      case TSqlstate.syntax_error_or_access_rule_violation_in_dynamic_statement._kiNO_SUBCLASS:
      {
        yClass = "syntax error or access rule violation in dynamic statement";
        break;
      }
      case TSqlstate.transaction_rollback._kiNO_SUBCLASS:
      case TSqlstate.transaction_rollback._kiINTEGRITY_CONSTRAINT_VIOLATION:
      case TSqlstate.transaction_rollback._kiSERIALIZATION_FAILURE:
      case TSqlstate.transaction_rollback._kiSTATEMENT_COMPLETION_UNKNOWN:
      {
        yClass = "transaction rollback";
        break;
      }
      case TSqlstate.triggered_data_change_violation._kiNO_SUBCLASS:
      {
        yClass = "triggered data change violation";
        break;
      }
      case TSqlstate.warning._kiNO_SUBCLASS:
      case TSqlstate.warning._kiCURSOR_OPERATION_CONFLICT:
      case TSqlstate.warning._kiDISCONNECT_ERROR:
      case TSqlstate.warning._kiIMPLICIT_ZERO_BIT_PADDING:
      case TSqlstate.warning._kiINSUFFICIENT_ITEM_DESCRIPTOR_AREAS:
      case TSqlstate.warning._kiNULL_VALUE_ELIMINATED_IN_SET_FUNCTION:
      case TSqlstate.warning._kiPRIVILEGE_NOT_GRANTED:
      case TSqlstate.warning._kiPRIVILEGE_NOT_REVOKED:
      case TSqlstate.warning._kiQUERY_EXPRESSION_TOO_LONG_FOR_INFORMATION_SCHEMA:
      case TSqlstate.warning._kiSEARCH_CONDITION_TOO_LONG_FOR_INFORMATION_SCHEMA:
      case TSqlstate.warning._kiSTRING_DATA_RIGHT_TRUNCATION:
      {
        yClass = "warning";
        break;
      }
      case TSqlstate.with_check_option_violation._kiNO_SUBCLASS:
      {
        yClass = "with check option violation";
        break;
      }
      default:
      {
        yClass = "unknown code";
      }
    }
    return yClass;
  
  }  // getClassValue()
      
  /**
  *  Gets  the  complete  description for the SQLSTATE as  in  the  source
  *  parameter \a kySQLSTATE.
  *  @return Complete description of the SQLSTATE in \a kySQLSTATE.
  */
  public String getDescription (final String kySQLSTATE)
  {

    StringBuffer   yDescription = new StringBuffer (getClassValue (kySQLSTATE));
  
    yDescription.append (": ");
    yDescription.append (getSubclassValue (kySQLSTATE));
    if ( TSqlstate.SQLMESSAGE.length() > 0 )
    {
      yDescription.append (" (");
      yDescription.append (TSqlstate.SQLMESSAGE);
      yDescription.append (")");
    }
    return yDescription.toString();

  }  // getDescription()

  /**
  *  Gets SQLSTATE subclass.
  *  @return Subclass description.
  */
  public final String getSubclassValue()
  {
    return getSubclassValue (TSqlstate.SQLSTATE);
  }

  /**
  *  Gets SQLSTATE subclass for \a kySQLSTATE.
  *  @return Subclass description.
  */
  public final String getSubclassValue (final String kySQLSTATE)
  {
  
    String   ySubclass = new String();
  
    switch (getUniqueValue (kySQLSTATE))
    {
      case TSqlstate.ambiguous_cursor_name._kiNO_SUBCLASS:
      case TSqlstate.cardinality_violation._kiNO_SUBCLASS:
      case TSqlstate.connection_exception._kiNO_SUBCLASS:
      case TSqlstate.data_exception._kiNO_SUBCLASS:
      case TSqlstate.dependent_privilege_descriptors_still_exist._kiNO_SUBCLASS:
      case TSqlstate.dynamic_sql_error._kiNO_SUBCLASS:
      case TSqlstate.feature_not_supported._kiNO_SUBCLASS:
      case TSqlstate.integrity_constraint_violation._kiNO_SUBCLASS:
      case TSqlstate.invalid_authorization_specification._kiNO_SUBCLASS:
      case TSqlstate.invalid_catalog_name._kiNO_SUBCLASS:
      case TSqlstate.invalid_character_set_name._kiNO_SUBCLASS:
      case TSqlstate.invalid_condition_number._kiNO_SUBCLASS:
      case TSqlstate.invalid_connection_name._kiNO_SUBCLASS:
      case TSqlstate.invalid_cursor_name._kiNO_SUBCLASS:
      case TSqlstate.invalid_cursor_state._kiNO_SUBCLASS:
      case TSqlstate.invalid_schema_name._kiNO_SUBCLASS:
      case TSqlstate.invalid_sql_descriptor_name._kiNO_SUBCLASS:
      case TSqlstate.invalid_sql_statement_name._kiNO_SUBCLASS:
      case TSqlstate.invalid_transaction_state._kiNO_SUBCLASS:
      case TSqlstate.invalid_transaction_termination._kiNO_SUBCLASS:
      case TSqlstate.no_data._kiNO_SUBCLASS:
      case TSqlstate.remote_database_access._kiNO_SUBCLASS:
      case TSqlstate.successful_completion._kiNO_SUBCLASS:
      case TSqlstate.syntax_error_or_access_rule_violation._kiNO_SUBCLASS:
      case TSqlstate.syntax_error_or_access_rule_violation_in_direct_statement._kiNO_SUBCLASS:
      case TSqlstate.syntax_error_or_access_rule_violation_in_dynamic_statement._kiNO_SUBCLASS:
      case TSqlstate.transaction_rollback._kiNO_SUBCLASS:
      case TSqlstate.triggered_data_change_violation._kiNO_SUBCLASS:
      case TSqlstate.warning._kiNO_SUBCLASS:
      case TSqlstate.with_check_option_violation._kiNO_SUBCLASS:
      {
        ySubclass = "(no subclass)";
        break;
      }
      case TSqlstate.connection_exception._kiCONNECTION_DOES_NOT_EXIST:
      {
        ySubclass = "connection does not exist";
        break;
      }
      case TSqlstate.connection_exception._kiCONNECTION_FAILURE:
      {
        ySubclass = "connection failure";
        break;
      }
      case TSqlstate.connection_exception._kiCONNECTION_NAME_IN_USE:
      {
        ySubclass = "connection name in use";
        break;
      }
      case TSqlstate.connection_exception._kiSQLCLIENT_UNABLE_TO_ESTABLISH_SQLCONNECTION:
      {
        ySubclass = "SQL-client unable to establish SQL-connection";
        break;
      }
      case TSqlstate.connection_exception._kiSQLSERVER_REJECTED_ESTABLISHMENT_OF_SQLCONNECTION:
      {
        ySubclass = "SQL-server rejected establishment of SQL-connection";
        break;
      }
      case TSqlstate.connection_exception._kiTRANSACTION_RESOLUTION_UNKNOWN:
      {
        ySubclass = "transaction resolution unknown";
        break;
      }
      case TSqlstate.data_exception._kiCHARACTER_NOT_IN_REPERTOIRE:
      {
        ySubclass = "character not in repertoire";
        break;
      }
      case TSqlstate.data_exception._kiDATETIME_FIELD_OVERFLOW:
      {
        ySubclass = "datetime field overflow";
        break;
      }
      case TSqlstate.data_exception._kiDIVISION_BY_ZERO:
      {
        ySubclass = "division by zero";
        break;
      }
      case TSqlstate.data_exception._kiERROR_IN_ASSIGNMENT:
      {
        ySubclass = "error in assignment";
        break;
      }
      case TSqlstate.data_exception._kiINDICATOR_OVERFLOW:
      {
        ySubclass = "indicator overflow";
        break;
      }
      case TSqlstate.data_exception._kiINTERVAL_FIELD_OVERFLOW:
      {
        ySubclass = "interval field overflow";
        break;
      }
      case TSqlstate.data_exception._kiINVALID_CHARACTER_VALUE_FOR_CAST:
      {
        ySubclass = "invalid character value for cast";
        break;
      }
      case TSqlstate.data_exception._kiINVALID_DATETIME_FORMAT:
      {
        ySubclass = "invalid datetime format";
        break;
      }
      case TSqlstate.data_exception._kiINVALID_ESCAPE_CHARACTER:
      {
        ySubclass = "invalid escape character";
        break;
      }
      case TSqlstate.data_exception._kiINVALID_ESCAPE_SEQUENCE:
      {
        ySubclass = "invalid escape sequence";
        break;
      }
      case TSqlstate.data_exception._kiINVALID_PARAMETER_VALUE:
      {
        ySubclass = "invalid parameter value";
        break;
      }
      case TSqlstate.data_exception._kiINVALID_TIME_ZONE_DISPLACEMENT_VALUE:
      {
        ySubclass = "invalid time zone displacement value";
        break;
      }
      case TSqlstate.data_exception._kiNULL_VALUE_NO_INDICATOR_PARAMETER:
      {
        ySubclass = "null value, no indicator parameter";
        break;
      }
      case TSqlstate.data_exception._kiNUMERIC_VALUE_OUT_OF_RANGE:
      {
        ySubclass = "numeric value out of range";
        break;
      }
      case TSqlstate.data_exception._kiSTRING_DATA_LENGTH_MISMATCH:
      {
        ySubclass = "string data, length mismatch";
        break;
      }
      case TSqlstate.data_exception._kiSTRING_DATA_RIGHT_TRUNCATION:
      {
        ySubclass = "string data, right truncation";
        break;
      }
      case TSqlstate.data_exception._kiSUBSTRING_ERROR:
      {
        ySubclass = "substring error";
        break;
      }
      case TSqlstate.data_exception._kiTRIM_ERROR:
      {
        ySubclass = "trim error";
        break;
      }
      case TSqlstate.data_exception._kiUNTERMINATED_C_STRING:
      {
        ySubclass = "unterminated C string";
        break;
      }
      case TSqlstate.dynamic_sql_error._kiCURSOR_SPECIFICATION_CANNOT_BE_EXECUTED:
      {
        ySubclass = "cursor specification cannot be executed";
        break;
      }
      case TSqlstate.dynamic_sql_error._kiINVALID_DESCRIPTOR_COUNT:
      {
        ySubclass = "invalid descriptor count";
        break;
      }
      case TSqlstate.dynamic_sql_error._kiINVALID_DESCRIPTOR_INDEX:
      {
        ySubclass = "invalid descriptor index";
        break;
      }
      case TSqlstate.dynamic_sql_error._kiPREPARED_STATEMENT_NOT_A_CURSOR_SPECIFICATION:
      {
        ySubclass = "prepared statement not a cursor specification";
        break;
      }
      case TSqlstate.dynamic_sql_error._kiRESTRICTED_DATA_TYPE_ATTRIBUTE_VIOLATION:
      {
        ySubclass = "restricted data type attribute violation";
        break;
      }
      case TSqlstate.dynamic_sql_error._kiUSING_CLAUSE_DOES_NOT_MATCH_DYNAMIC_PARAMETER_SPEC:
      {
        ySubclass = "using clause does not match dynamic parameter specifications";
        break;
      }
      case TSqlstate.dynamic_sql_error._kiUSING_CLAUSE_DOES_NOT_MATCH_TARGET_SPEC:
      {
        ySubclass = "using clause does not match target specifications";
        break;
      }
      case TSqlstate.dynamic_sql_error._kiUSING_CLAUSE_REQUIRED_FOR_DYNAMIC_PARAMETERS:
      {
        ySubclass = "using clause required for dynamic parameters";
        break;
      }
      case TSqlstate.dynamic_sql_error._kiUSING_CLAUSE_REQUIRED_FOR_RESULT_FIELDS:
      {
        ySubclass = "using clause required for result fields";
        break;
      }
      case TSqlstate.feature_not_supported._kiMULTIPLE_ENVIRONMENT_TRANSACTIONS:
      {
        ySubclass = "multiple environment transactions";
        break;
      }
      case TSqlstate.transaction_rollback._kiINTEGRITY_CONSTRAINT_VIOLATION:
      {
        ySubclass = "integrity constraint violation";
        break;
      }
      case TSqlstate.transaction_rollback._kiSERIALIZATION_FAILURE:
      {
        ySubclass = "serialization failure";
        break;
      }
      case TSqlstate.transaction_rollback._kiSTATEMENT_COMPLETION_UNKNOWN:
      {
        ySubclass = "statement completion unknown";
        break;
      }
      case TSqlstate.warning._kiCURSOR_OPERATION_CONFLICT:
      {
        ySubclass = "cursor operation conflict";
        break;
      }
      case TSqlstate.warning._kiDISCONNECT_ERROR:
      {
        ySubclass = "disconnect error";
        break;
      }
      case TSqlstate.warning._kiIMPLICIT_ZERO_BIT_PADDING:
      {
        ySubclass = "implicit zero-bit padding";
        break;
      }
      case TSqlstate.warning._kiINSUFFICIENT_ITEM_DESCRIPTOR_AREAS:
      {
        ySubclass = "insufficient item descriptor areas";
        break;
      }
      case TSqlstate.warning._kiNULL_VALUE_ELIMINATED_IN_SET_FUNCTION:
      {
        ySubclass = "null value eliminated in set function";
        break;
      }
      case TSqlstate.warning._kiPRIVILEGE_NOT_GRANTED:
      {
        ySubclass = "privilege not granted";
        break;
      }
      case TSqlstate.warning._kiPRIVILEGE_NOT_REVOKED:
      {
        ySubclass = "privilege not revoked";
        break;
      }
      case TSqlstate.warning._kiQUERY_EXPRESSION_TOO_LONG_FOR_INFORMATION_SCHEMA:
      {
        ySubclass = "query expression too long for information schema";
        break;
      }
      case TSqlstate.warning._kiSEARCH_CONDITION_TOO_LONG_FOR_INFORMATION_SCHEMA:
      {
        ySubclass = "search condition too long for information schema";
        break;
      }
      case TSqlstate.warning._kiSTRING_DATA_RIGHT_TRUNCATION:
      {
        ySubclass = "string data, right truncation";
        break;
      }
      default:
      {
        ySubclass = "unknown code";
      }
    }
    return ySubclass;
  
  }  // getSubclassValue()

}  // class TSqlstateDescriptor
