/*
*  Name:      defs.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   UESQLC definitions
*  Date:      $Date: 2002/08/19 12:59:57 $
*  Revision:  $Revision: 1.2 $
*
*  Copyright (C) 1996-2002  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _UESQLC_DEFS_UESQLC__
#define _UESQLC_DEFS_UESQLC__

#include <mpcl/error_exception.hh>
#include <mpcl/general-exceptions.hh>
#include <mpcl/local_date.hh>
#include <mpcl/map.hh>
#include <mpcl/string.hh>
#include <mpcl/warning_exception.hh>


///  Name-space for UESQL.
namespace uesql
{
  
  //
  //  E X P O R T E D   V A R I A B L E S
  //

  //
  //  These variables are used in sources generated by UESQLC.
  //

  /**
  *  SQL state parameter (values are implementation defined and using it
  *  is deprecated).
  */
  extern long int   SQLCODE;

  /**
  *  SQL extended description (target dependent). This an extended message
  *  for including target dependent description of errors.
  */
  extern mpcl::TString   SQLMESSAGE;

  ///  SQL state parameter (values in clause 22, "Status codes").
  extern mpcl::TString   SQLSTATE;

  //
  //  SQLSTATE class and subclass values.
  //    Classes has been represented as a C++ namespace, and subclasses have
  //    been represented as a constant pointer to a zero terminated string.
  //

  ///  Condition: ambiguous cursor name.
  namespace ambiguous_cursor_name
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS
    };
  }

  ///  Condition: cardinality violation.
  namespace cardinality_violation
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 1000
    };
  }
  
  ///  Condition: connection exception.
  namespace connection_exception
  {
    extern const char*   pkcNO_SUBCLASS;
    extern const char*   pkcCONNECTION_DOES_NOT_EXIST;
    extern const char*   pkcCONNECTION_FAILURE;
    extern const char*   pkcCONNECTION_NAME_IN_USE;
    extern const char*   pkcSQLCLIENT_UNABLE_TO_ESTABLISH_SQLCONNECTION;
    extern const char*   pkcSQLSERVER_REJECTED_ESTABLISHMENT_OF_SQLCONNECTION;
    extern const char*   pkcTRANSACTION_RESOLUTION_UNKNOWN;
    enum EValues
    {
      eNO_SUBCLASS = 2000                                ,
      eCONNECTION_DOES_NOT_EXIST                         ,
      eCONNECTION_FAILURE                                ,
      eCONNECTION_NAME_IN_USE                            ,
      eSQLCLIENT_UNABLE_TO_ESTABLISH_SQLCONNECTION       ,
      eSQLSERVER_REJECTED_ESTABLISHMENT_OF_SQLCONNECTION ,
      eTRANSACTION_RESOLUTION_UNKNOWN
    };
  }
  
  ///  Condition: data exception.
  namespace data_exception
  {
    extern const char*   pkcNO_SUBCLASS;
    extern const char*   pkcCHARACTER_NOT_IN_REPERTOIRE;
    extern const char*   pkcDATETIME_FIELD_OVERFLOW;
    extern const char*   pkcDIVISION_BY_ZERO;
    extern const char*   pkcERROR_IN_ASSIGNMENT;
    extern const char*   pkcINDICATOR_OVERFLOW;
    extern const char*   pkcINTERVAL_FIELD_OVERFLOW;
    extern const char*   pkcINVALID_CHARACTER_VALUE_FOR_CAST;
    extern const char*   pkcINVALID_DATETIME_FORMAT;
    extern const char*   pkcINVALID_ESCAPE_CHARACTER;
    extern const char*   pkcINVALID_ESCAPE_SEQUENCE;
    extern const char*   pkcINVALID_PARAMETER_VALUE;
    extern const char*   pkcINVALID_TIME_ZONE_DISPLACEMENT_VALUE;
    extern const char*   pkcNULL_VALUE_NO_INDICATOR_PARAMETER;
    extern const char*   pkcNUMERIC_VALUE_OUT_OF_RANGE;
    extern const char*   pkcSTRING_DATA_LENGTH_MISMATCH;
    extern const char*   pkcSTRING_DATA_RIGHT_TRUNCATION;
    extern const char*   pkcSUBSTRING_ERROR;
    extern const char*   pkcTRIM_ERROR;
    extern const char*   pkcUNTERMINATED_C_STRING;
    enum EValues
    {
      eNO_SUBCLASS = 3000                   ,
      eCHARACTER_NOT_IN_REPERTOIRE          ,
      eDATETIME_FIELD_OVERFLOW              ,
      eDIVISION_BY_ZERO                     ,
      eERROR_IN_ASSIGNMENT                  ,
      eINDICATOR_OVERFLOW                   ,
      eINTERVAL_FIELD_OVERFLOW              ,
      eINVALID_CHARACTER_VALUE_FOR_CAST     ,
      eINVALID_DATETIME_FORMAT              ,
      eINVALID_ESCAPE_CHARACTER             ,
      eINVALID_ESCAPE_SEQUENCE              ,
      eINVALID_PARAMETER_VALUE              ,
      eINVALID_TIME_ZONE_DISPLACEMENT_VALUE ,
      eNULL_VALUE_NO_INDICATOR_PARAMETER    ,
      eNUMERIC_VALUE_OUT_OF_RANGE           ,
      eSTRING_DATA_LENGTH_MISMATCH          ,
      eSTRING_DATA_RIGHT_TRUNCATION         ,
      eSUBSTRING_ERROR                      ,
      eTRIM_ERROR                           ,
      eUNTERMINATED_C_STRING
    };
  }
  
  ///  Condition: dependent privilege descriptors still exist.
  namespace dependent_privilege_descriptors_still_exist
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 4000
    };
  }
  
  ///  Condition: dynamic SQL error.
  namespace dynamic_sql_error
  {
    extern const char*   pkcNO_SUBCLASS;
    extern const char*   pkcCURSOR_SPECIFICATION_CANNOT_BE_EXECUTED;
    extern const char*   pkcINVALID_DESCRIPTOR_COUNT;
    extern const char*   pkcINVALID_DESCRIPTOR_INDEX;
    extern const char*   pkcPREPARED_STATEMENT_NOT_A_CURSOR_SPECIFICATION;
    extern const char*   pkcRESTRICTED_DATA_TYPE_ATTRIBUTE_VIOLATION;
    extern const char*   pkcUSING_CLAUSE_DOES_NOT_MATCH_DYNAMIC_PARAMETER_SPEC;
    extern const char*   pkcUSING_CLAUSE_DOES_NOT_MATCH_TARGET_SPEC;
    extern const char*   pkcUSING_CLAUSE_REQUIRED_FOR_DYNAMIC_PARAMETERS;
    extern const char*   pkcUSING_CLAUSE_REQUIRED_FOR_RESULT_FIELDS;
    enum EValues
    {
      eNO_SUBCLASS = 5000                                 ,
      eCURSOR_SPECIFICATION_CANNOT_BE_EXECUTED            ,
      eINVALID_DESCRIPTOR_COUNT                           ,
      eINVALID_DESCRIPTOR_INDEX                           ,
      ePREPARED_STATEMENT_NOT_A_CURSOR_SPECIFICATION      ,
      eRESTRICTED_DATA_TYPE_ATTRIBUTE_VIOLATION           ,
      eUSING_CLAUSE_DOES_NOT_MATCH_DYNAMIC_PARAMETER_SPEC ,
      eUSING_CLAUSE_DOES_NOT_MATCH_TARGET_SPEC            ,
      eUSING_CLAUSE_REQUIRED_FOR_DYNAMIC_PARAMETERS       ,
      eUSING_CLAUSE_REQUIRED_FOR_RESULT_FIELDS
    };
  }
  
  ///  Condition: feature not supported.
  namespace feature_not_supported
  {
    extern const char*   pkcNO_SUBCLASS;
    extern const char*   pkcMULTIPLE_ENVIRONMENT_TRANSACTIONS;
    enum EValues
    {
      eNO_SUBCLASS = 6000                ,
      eMULTIPLE_ENVIRONMENT_TRANSACTIONS
    };
  }
  
  ///  Condition: integrity constraint violation.
  namespace integrity_constraint_violation
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 7000
    };
  }
  
  ///  Condition: invalid authorization specification.
  namespace invalid_authorization_specification
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 8000
    };
  }
  
  ///  Condition: invalid catalog name.
  namespace invalid_catalog_name
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 9000
    };
  }
  
  ///  Condition: invalid character set name.
  namespace invalid_character_set_name
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 10000
    };
  }
  
  ///  Condition: invalid condition number.
  namespace invalid_condition_number
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 11000
    };
  }
  
  ///  Condition: invalid connection name.
  namespace invalid_connection_name
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 12000
    };
  }
  
  ///  Condition: invalid cursor name.
  namespace invalid_cursor_name
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 13000
    };
  }
  
  ///  Condition: invalid cursor state.
  namespace invalid_cursor_state
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 14000
    };
  }
  
  ///  Condition: invalid schema name.
  namespace invalid_schema_name
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 15000
    };
  }
  
  ///  Condition: invalid SQL descriptor name.
  namespace invalid_sql_descriptor_name
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 16000
    };
  }
  
  ///  Condition: invalid SQL statement name.
  namespace invalid_sql_statement_name
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 17000
    };
  }
  
  ///  Condition: invalid transaction state.
  namespace invalid_transaction_state
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 18000
    };
  }
  
  ///  Condition: invalid transaction termination.
  namespace invalid_transaction_termination
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 19000
    };
  }
  
  ///  Condition: no data.
  namespace no_data
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 20000
    };
  }
  
  ///  Condition: remote database access.
  namespace remote_database_access
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 21000
    };
  }
  
  ///  Condition: successful completion.
  namespace successful_completion
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 22000
    };
  }
  
  ///  Condition: syntax error or access rule violation.
  namespace syntax_error_or_access_rule_violation
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 23000
    };
  }
  
  ///  Condition: syntax error or access rule violation in direct statement.
  namespace syntax_error_or_access_rule_violation_in_direct_statement
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 24000
    };
  }
  
  ///  Condition: syntax error or access rule violation in dynamic statement.
  namespace syntax_error_or_access_rule_violation_in_dynamic_statement
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 25000
    };
  }
  
  ///  Condition: transaction rollback.
  namespace transaction_rollback
  {
    extern const char*   pkcNO_SUBCLASS;
    extern const char*   pkcINTEGRITY_CONSTRAINT_VIOLATION;
    extern const char*   pkcSERIALIZATION_FAILURE;
    extern const char*   pkcSTATEMENT_COMPLETION_UNKNOWN;
    enum EValues
    {
      eNO_SUBCLASS = 26000            ,
      eINTEGRITY_CONSTRAINT_VIOLATION ,
      eSERIALIZATION_FAILURE          ,
      eSTATEMENT_COMPLETION_UNKNOWN
    };
  }
  
  ///  Condition: triggered data change violation.
  namespace triggered_data_change_violation
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 27000
    };
  }
  
  ///  Condition: warning.
  namespace warning
  {
    extern const char*   pkcNO_SUBCLASS;
    extern const char*   pkcCURSOR_OPERATION_CONFLICT;
    extern const char*   pkcDISCONNECT_ERROR;
    extern const char*   pkcIMPLICIT_ZERO_BIT_PADDING;
    extern const char*   pkcINSUFFICIENT_ITEM_DESCRIPTOR_AREAS;
    extern const char*   pkcNULL_VALUE_ELIMINATED_IN_SET_FUNCTION;
    extern const char*   pkcPRIVILEGE_NOT_GRANTED;
    extern const char*   pkcPRIVILEGE_NOT_REVOKED;
    extern const char*   pkcQUERY_EXPRESSION_TOO_LONG_FOR_INFORMATION_SCHEMA;
    extern const char*   pkcSEARCH_CONDITION_TOO_LONG_FOR_INFORMATION_SCHEMA;
    extern const char*   pkcSTRING_DATA_RIGHT_TRUNCATION;
    enum EValues
    {
      eNO_SUBCLASS = 28000                              ,
      eCURSOR_OPERATION_CONFLICT                        ,
      eDISCONNECT_ERROR                                 ,
      eIMPLICIT_ZERO_BIT_PADDING                        ,
      eINSUFFICIENT_ITEM_DESCRIPTOR_AREAS               ,
      eNULL_VALUE_ELIMINATED_IN_SET_FUNCTION            ,
      ePRIVILEGE_NOT_GRANTED                            ,
      ePRIVILEGE_NOT_REVOKED                            ,
      eQUERY_EXPRESSION_TOO_LONG_FOR_INFORMATION_SCHEMA ,
      eSEARCH_CONDITION_TOO_LONG_FOR_INFORMATION_SCHEMA ,
      eSTRING_DATA_RIGHT_TRUNCATION
    };
  }
  
  ///  Condition: with check option violation.
  namespace with_check_option_violation
  {
    extern const char*   pkcNO_SUBCLASS;
    enum EValues
    {
      eNO_SUBCLASS = 29000
    };
  }

  ///  Base SQL data type. It is used as base class for embedded variables.
  class TBaseType
  {

    protected:

      ///  If true, then data retrieved from SQL server, is null.
      bool   gIsNull;


    public:
      
      //
      //  C O N S T R U C T O R S
      //

      ///  Builds a new instance.
      TBaseType (void) :
        gIsNull (true) {}

      ///  Builds a new instance from another instance.
      TBaseType (const TBaseType& rktSQL_DATA_TYPE) :
        gIsNull (rktSQL_DATA_TYPE.gIsNull)          {}

      ///  Destroys the instance.
      virtual ~TBaseType (void) {}
      
      /**
      *  Returns a buffer where you can assign a value of, at most,
      *  \a kzSIZE octets.
      *  @param kzSIZE Octet count.
      *  @return A pointer to the buffer.
      */
      virtual void* getBuffer (const std::size_t kzSIZE) = 0;

      /**
      *  Returns a buffer where you can assign a value of, at most,
      *  \a kzSIZE octets.
      *  @param kzSIZE Octet count.
      *  @return A pointer to the buffer.
      */
      virtual void updateFromBuffer (void) = 0;

      /**
      *  Assigns value \a pkcVALUE.
      *  @param pkcVALUE Source value.
      */
      virtual void set (const char* pkcVALUE) = 0;
      
      /**
      *  Assigns value \a rkyVALUE.
      *  @param rkyVALUE Source value.
      */
      virtual void set (const std::string& rkyVALUE)
      {
        set (rkyVALUE.c_str());
      }


    public:

      //
      //  S E L E C T O R S
      //

      /**
      *  Returns a string with the value of the instance. If the instance holds
      *  a null value, then the string 'NULL' will be printed.
      *  @return The result of converting this instance to string.
      */
      virtual mpcl::TString get (void) const = 0;

      /**
      *  Returns true if the last data retrieved has been a NULL value.
      *  @return True if the last data retrieved has been a NULL value.
      */
      bool isNull (void) const
      {
        return gIsNull;
      }

  };  // class TBaseType

  ///  SQL date handler.
  class TDate               :
    public mpcl::TLocalDate ,
    public TBaseType
  {

    private:
      
      ///  Buffer for getting values passively.
      mutable char*   pcBuffer;


    public:

      //
      //  C O N S T R U C T O R S
      //

      ///  Builds a new instance.
      TDate (void)              :
        mpcl::TLocalDate ()     ,
        TBaseType        ()     ,
        pcBuffer         (NULL) {}

      ///  Builds a new instance from another instance.
      TDate (const TDate& rktSQL_DATE) :
        mpcl::TLocalDate (rktSQL_DATE) ,
        TBaseType        (rktSQL_DATE) ,
        pcBuffer         (NULL)        {}

      TDate (const EDateType keDATE_TYPE) :
        mpcl::TLocalDate (keDATE_TYPE)    ,
        TBaseType        ()               ,
        pcBuffer         (NULL)
      {
        gIsNull = false;
      }

      TDate (const int kiYEAR, const int kiMONTH, const int kiDAY) :
        mpcl::TLocalDate (kiYEAR, kiMONTH, kiDAY)                  ,
        TBaseType        ()                                        ,
        pcBuffer         (NULL)
      {
        gIsNull = false;
      }

      TDate (const char* pkcDATE);

      TDate (const std::string& rkyDATE);

      ///  Destroys the instance.
      virtual ~TDate (void);

      /**
      *  Reserves  a  buffer \a kzSIZE  octets  length,  to  let somebody store
      *  a date as a zero-terminated string of chars.
      *  @param kzSIZE Size of the buffer.
      *  @return The buffer.
      */
      void* getBuffer (const std::size_t kzSIZE);

      /**
      *  Assigns value \a pkcVALUE.  If \a pkcVALUE is null or points to a zero
      *  length string, then the instance is null.
      *  @pre ( !pkcVALUE || !*pkcVALUE ) => isNull()
      *  @param pkcVALUE Source value.
      */
      void set (const char* pkcVALUE);

      /**
      *  Assigns value \a rkyVALUE.  If \a rkyVALUE is empty, then the instance
      *  is null.
      *  @pre ( rkyVALUE.empty() ) => isNull()
      *  @param rkyVALUE Source value.
      */
      void set (const std::string& rkyVALUE)
      {
        set (rkyVALUE.c_str());
      }

      void updateFromBuffer (void);


    public:

      //
      //  S E L E C T O R S
      //

      mpcl::TString get (void) const
      {
        mpcl::TString   yResult;

        if ( gIsNull )
        {
          yResult = "null";
        }
        else
        {
          yResult = TLocalDate::get ("'%Y-%m-%d'");
        }
        return yResult;
      }

      /**
      *  Returns date in SQL format.
      *  @param pkcDATE_FORMAT Date format in strftime-like format. It defaults
      *                        to ANSI SQL date  format.  A  valid date  can be
      *                        '1970-12-31'.
      *  @return Date in readable format.
      *  @warning pkcDATE_FORMAT must include the enclosing quotes (').
      */
      mpcl::TString get (const char* pkcDATE_FORMAT) const
      {
        mpcl::TString   yResult;

        if ( gIsNull )
        {
          yResult = "null";
        }
        else
        {
          yResult = TLocalDate::get (pkcDATE_FORMAT);
        }
        return yResult;
      }

  };  // class TDate

  ///  SQL type INTEGER.
  class TInteger : public TBaseType
  {

    private:

      ///  Integer value.
      long int   liValue;

      ///  Buffer for getting values passively.
      mutable char*   pcBuffer;


    public:

      //
      //  C O N S T R U C T O R S
      //

      ///  Builds a new instance.
      TInteger (void)    :
        TBaseType ()     ,
        liValue   (0L)   ,
        pcBuffer  (NULL) {}

      ///  Builds a new instance from another instance.
      TInteger (const TInteger& rktSQL_INTEGER) :
        TBaseType (rktSQL_INTEGER)              ,
        liValue   (rktSQL_INTEGER.liValue)      ,
        pcBuffer  (NULL)                        {}

      TInteger (const long int kliVALUE) :
        TBaseType ()                     ,
        liValue   (kliVALUE)             ,
        pcBuffer  (NULL)
      {
        gIsNull = false; 
      }

      TInteger (const char* pkcSQL_INTEGER) :
        TBaseType ()                        ,
        liValue   (0L)                      ,
        pcBuffer  (NULL)
      {
        set (pkcSQL_INTEGER);
      }

      TInteger (const std::string& rkySQL_INTEGER) :
        TBaseType ()                               ,
        liValue   (0L)                             ,
        pcBuffer  (NULL)
      {
        set (rkySQL_INTEGER);
      }

      ///  Destroys the instance.
      virtual ~TInteger (void);
      
      void* getBuffer (const std::size_t kzSIZE);

      void updateFromBuffer (void);

      /**
      *  Assigns value \a pkcVALUE.  If \a pkcVALUE is null or points to a zero
      *  length string, then the instance is null.
      *  @pre ( !pkcVALUE || !*pkcVALUE ) => isNull()
      *  @param pkcVALUE Source value.
      */
      void set (const char* pkcVALUE);
      
      /**
      *  Assigns value \a rkyVALUE.  If \a rkyVALUE is empty, then the instance
      *  is null.
      *  @pre ( rkyVALUE.empty() ) => isNull()
      *  @param rkyVALUE Source value.
      */
      void set (const std::string& rkyVALUE)
      {
        set (rkyVALUE.c_str());
      }


    public:

      //
      //  S E L E C T O R S
      //

      /**
      *  Returns a string.
      *  @return The result of converting this instance to string.
      */
      mpcl::TString get (void) const
      {
        mpcl::TString   yResult;

        if ( gIsNull )
        {
          yResult = "null";
        }
        else
        {
          yResult = mpcl::Format ("%ld", liValue);
        }
        return yResult;
      }

      ///  Returns the long integer value.
      operator long int (void) const
      {
        return liValue;
      }

  };  // class TInteger

  ///  SQL type REAL.
  class TReal : public TBaseType
  {

    private:

      ///  Real value.
      double   dValue;

      ///  Buffer for getting values passively.
      mutable char*   pcBuffer;


    public:

      //
      //  C O N S T R U C T O R S
      //

      TReal (void)       :
        TBaseType ()     ,
        dValue    (0L)   ,
        pcBuffer  (NULL) {}

      ///  Builds a new instance from another instance.
      TReal (const TReal& rktSQL_REAL) :
        TBaseType (rktSQL_REAL)        ,
        dValue    (rktSQL_REAL.dValue) ,
        pcBuffer  (NULL)               {}

      TReal (const double kdVALUE) :
        TBaseType ()               ,
        dValue    (kdVALUE)        ,
        pcBuffer  (NULL)
      {
        gIsNull = false;
      }

      TReal (const char* pkcSQL_REAL) :
        TBaseType ()                  ,
        dValue    (0)                 ,
        pcBuffer  (NULL)
      {
        set (pkcSQL_REAL);
      }

      TReal (const std::string& rkySQL_REAL) :
        TBaseType ()                         ,
        dValue    (0)                        ,
        pcBuffer  (NULL)
      {
        set (rkySQL_REAL);
      }

      virtual ~TReal (void);
      
      void* getBuffer (const std::size_t kzSIZE);

      void updateFromBuffer (void);

      /**
      *  Assigns value \a pkcVALUE.  If \a pkcVALUE is null or points to a zero
      *  length string, then the instance is null.
      *  @pre ( !pkcVALUE || !*pkcVALUE ) => isNull()
      *  @param pkcVALUE Source value.
      */
      void set (const char* pkcVALUE);
      
      /**
      *  Assigns value \a rkyVALUE.  If \a rkyVALUE is empty, then the instance
      *  is null.
      *  @pre ( rkyVALUE.empty() ) => isNull()
      *  @param rkyVALUE Source value.
      */
      void set (const std::string& rkyVALUE)
      {
        set (rkyVALUE.c_str());
      }


    public:

      //
      //  S E L E C T O R S
      //

      /**
      *  Returns a string.
      *  @return The result of converting this instance to string.
      */
      mpcl::TString get (void) const
      {
        mpcl::TString   yResult;

        if ( gIsNull )
        {
          yResult = "null";
        }
        else
        {
          yResult = mpcl::Format ("%f", dValue);
        }
        return yResult;
      }

      /**
      *  Returns a string.
      *  @param pkcFORMAT Format string for converting from double to string.
      *  @return The result of converting this instance to string.
      *  @see std::printf(), mpcl::Format().
      */
      mpcl::TString get (const char* pkcFORMAT) const
      {
        mpcl::TString   yResult;

        if ( gIsNull )
        {
          yResult = "null";
        }
        else
        {
          yResult = mpcl::Format (pkcFORMAT, dValue);
        }
        return yResult;
      }

      ///  Returns the SQL real value.
      operator long double (void) const
      {
        return dValue;
      }

  };  // class TReal

  ///  SQL type string.
  class TString          :
    public TBaseType     ,
    public mpcl::TString
  {

    private:
      
      ///  Buffer for getting values passively.
      mutable char*   pcBuffer;
      

    public:

      ///  Import this type.
      typedef
        mpcl::TString::size_type
        size_type;


    public:
      
      //
      //  C O N S T R U C T O R S
      //
      
      explicit TString (void) :
        TBaseType     ()      ,
        mpcl::TString ()      ,
        pcBuffer      (NULL)  {}

      ///  Builds a new instance from another instance.
      TString (const TString& rkySQL_STRING) :
        TBaseType     (rkySQL_STRING)        ,
        mpcl::TString (rkySQL_STRING)        ,
        pcBuffer      (NULL)                 {}

      TString ( const TString& rkySQL_STRING  ,
                size_type      zOFFSET        ,
                size_type      zLENGTH = npos )         :
        TBaseType  (rkySQL_STRING)                      ,
        mpcl::TString (rkySQL_STRING, zOFFSET, zLENGTH) ,
        pcBuffer      (NULL)                            {}

      TString (const mpcl::TString& rkySOURCE) :
        TBaseType     ()                       ,
        mpcl::TString (rkySOURCE)              ,
        pcBuffer      (NULL)
      {
        gIsNull = false;
      }

      /**
      *  Extends the parent constructor with support for NULL strings.
      *  @param pkcVALUE Source string.
      *  @param zLENGTH  Source string length.
      */
      TString (const char* pkcVALUE, size_type zLENGTH) :
        TBaseType     ()                                ,
        mpcl::TString ()                                ,
        pcBuffer      (NULL)
      {
        if ( pkcVALUE )
        {
          gIsNull = false;
          assign (pkcVALUE, zLENGTH);
        }
      }
      
      /**
      *  Extends the parent constructor with support for NULL strings.
      *  @param pkcVALUE Source string.
      *  @exception TExceptionEL_Constraint If source string is NULL.
      */
      TString (const char* pkcVALUE) :
        TBaseType     ()             ,
        mpcl::TString ()             ,
        pcBuffer      (NULL)
      {
        if ( pkcVALUE )
        {
          gIsNull = false;
          assign (pkcVALUE);
        }
      }
      
      TString (size_type zTIMES, char cSOURCE) :
        TBaseType     ()                       ,
        mpcl::TString (zTIMES, cSOURCE)        ,
        pcBuffer      (NULL)
      {
        gIsNull = false;
      }
      
      template<typename InputIterator>
      TString (InputIterator tBEGIN_ITER, InputIterator tEND_ITER) :
        TBaseType     ()                                           ,
        mpcl::TString (tBEGIN_ITER, tEND_ITER)                     ,
        pcBuffer      (NULL)
      {
        gIsNull = false;
      }

      ~TString (void);
      
      TString& operator = (const TString& rkySQL_STRING)
      {
        mpcl::TString::operator = (rkySQL_STRING);
        gIsNull                 = rkySQL_STRING.gIsNull;
        return *this;
      }

      TString& operator = (const mpcl::TString& rkySOURCE)
      {
        mpcl::TString::operator = (rkySOURCE);
        gIsNull                 = false;
        return *this;
      }

      TString& operator = (const char* pkcVALUE)
      {
        if ( !pkcVALUE )
        {
          throw mpcl::TExceptionEL_Constraint ("null string", __FILE__, __LINE__);
        }
        mpcl::TString::operator = (pkcVALUE);
        gIsNull                 = false;
        return *this;
      }

      TString& operator = (char cSOURCE)
      {
        mpcl::TString::operator = (cSOURCE);
        gIsNull                 = false;
        return *this;
      }

      void* getBuffer (const std::size_t kzSIZE);

      /**
      *  Assigns value \a pkcVALUE.  If \a pkcVALUE is null,  then the instance
      *  is null.
      *  @pre ( !pkcVALUE ) => isNull()
      *  @param pkcVALUE Source value.
      */
      void set (const char* pkcVALUE)
      {
        gIsNull = ( !pkcVALUE );
        if ( gIsNull )
        {
          erase();
        }
        else
        {
          assign (pkcVALUE);
        }
      }

      /**
      *  Assigns value \a rkyVALUE.
      *  @param rkyVALUE Source value.
      */
      void set (const std::string& rkyVALUE)
      {
        assign (rkyVALUE);
        gIsNull = false;
      }

      void updateFromBuffer (void);


    public:

      //
      //  S E L E C T O R S
      //

      /**
      *  Returns a string.
      *  @return The result of converting this instance to string.
      */
      mpcl::TString get (void) const
      {
        mpcl::TString   yResult;

        if ( gIsNull )
        {
          yResult = "null";
        }
        else
        {
          yResult = '\'';
          yResult.append (*this);
          yResult.append (1, '\'');
        }
        return yResult;
      }

  };  // class TString

  ///  SQL type for SQL identifiers.
  class TIdentifier : public TString
  {

    public:
      
      //
      //  C O N S T R U C T O R S
      //
      
      explicit TIdentifier (void) : TString() {}

      ///  Builds a new instance from another instance.
      TIdentifier (const TIdentifier& rkySQL_IDENTIFIER) :
        TString (rkySQL_IDENTIFIER)                      {}

      TIdentifier ( const TIdentifier& rkySQL_IDENTIFIER ,
                    size_type          zOFFSET           ,
                    size_type          zLENGTH = npos    ) :
        TString (rkySQL_IDENTIFIER, zOFFSET, zLENGTH)      {}

      TIdentifier (const mpcl::TString& rkySOURCE) : TString (rkySOURCE) {}

      /**
      *  Extends the parent constructor with support for NULL strings.
      *  @param pkcVALUE Source string.
      *  @param zLENGTH  Source string length.
      */
      TIdentifier (const char* pkcVALUE, size_type zLENGTH) :
        TString (pkcVALUE, zLENGTH)                         {}
      
      /**
      *  Extends the parent constructor with support for NULL strings.
      *  @param pkcVALUE Source string.
      *  @exception TExceptionEL_Constraint If source string is NULL.
      */
      TIdentifier (const char* pkcVALUE) : TString (pkcVALUE) {}

      TIdentifier (size_type zTIMES, char cSOURCE) :
        TString (zTIMES, cSOURCE)                  {}
      
      template<typename InputIterator>
      TIdentifier (InputIterator tBEGIN_ITER, InputIterator tEND_ITER) :
        TString (tBEGIN_ITER, tEND_ITER)                               {}


    public:

      //
      //  S E L E C T O R S
      //

      /**
      *  Returns a string.
      *  @return The result of converting this instance to string without quotes.
      */
      mpcl::TString get (void) const
      {
        return ( gIsNull ) ? "null" : *this;
      }
      
  };  // class TIdentifier

  ///  Handler for SQLSTATE status parameter.
  class TSqlstateDescriptor
  {

    private:
      
      /**
      *  Map with SQLSTATE values (strings) to integer translations.
      *  So the integer values are suitable for a \b switch statement.
      */
      static mpcl::TMap<mpcl::TString, int>   _tValueToIntMap;


    protected:
      
      //
      //  L O C A L   F U N C T I O N S
      //

      /**
      *  Adds a new pair (SQLSTATE, iVALUE) to the map.
      *  @param pkcSQLSTATE SQL state description.
      *  @param iVALUE      SQL state unique integer value.
      */
      static void _addCondition ( const char* pkcSQLSTATE ,
                                  int         iVALUE      );

      /**
      *  Initializes the SQLSTATE to integer map.
      */
      static void _initialize (void);
      
      
    public:

      //
      //  C O N S T R U C T O R S
      //

      ///  Builds a new instance and initializes the map.
      TSqlstateDescriptor (void)
      {
        _initialize();
      }


    public:

      //
      //  S E L E C T O R S
      //

      /**
      *  Returns true if the query is empty.
      *  @return True if query is empty.
      */
      bool queryIsEmpty (void) const
      {
        return ( getUniqueValue() == no_data::eNO_SUBCLASS );
      }

      /**
      *  Returns true if the query has only a single row.
      *  @return True if query has a single row.
      */
      bool queryHasSingleRow (void) const
      {
        //
        //  Check if query has generated only *one* row.
        //
        return ( ( getUniqueValue() != cardinality_violation::eNO_SUBCLASS ) &&
                 ( getUniqueValue() != no_data::eNO_SUBCLASS )                );
      }

      /**
      *  Finds the integer value for the current SQLSTATE.
      *  @return The integer value for the current SQLSTATE.
      */
      int getUniqueValue (void) const
      {
        int                                              iUniqueValue = -1;
        mpcl::TMap<mpcl::TString, int>::const_iterator   I            = _tValueToIntMap.find (SQLSTATE);

        if ( I != _tValueToIntMap.end() )
        {
          iUniqueValue = I->second;
        }        
        return iUniqueValue;
      }

      /**
      *  Finds the integer value for the SQLSTATE in \a pkcSQLSTATE.
      *  @return The integer value for the SQLSTATE in \a pkcSQLSTATE.
      */
      int getUniqueValue (const char* pkcSQLSTATE) const
      {
        return _tValueToIntMap [pkcSQLSTATE];
      }

      /**
      *  Gets SQLSTATE class.
      *  @return Class description.
      */
      const char* getClassValue (void) const
      {
        return getClassValue (SQLSTATE.c_str());
      }
      
      /**
      *  Gets SQLSTATE class for \a pkcSQLSTATE.
      *  @return Class description.
      */
      const char* getClassValue (const char* pkcSQLSTATE) const;
      
      /**
      *  Gets  the  complete  description for the SQLSTATE as  in  the  source
      *  parameter \a pkcSQLSTATE.
      *  @return Complete description of the SQLSTATE in \a pkcSQLSTATE.
      */
      mpcl::TString getDescription (const char* pkcSQLSTATE) const;
      
      /**
      *  Gets SQLSTATE subclass.
      *  @return Subclass description.
      */
      const char* getSubclassValue (void) const
      {
        return getSubclassValue (SQLSTATE.c_str());
      }
      
      /**
      *  Gets SQLSTATE subclass for \a pkcSQLSTATE.
      *  @return Subclass description.
      */
      const char* getSubclassValue (const char* pkcSQLSTATE) const;

  };  // class TSqlstateDescriptor


  ///  Exception: SQL exception class for NOT FOUND condition.
  class TExceptionUESQL_NotFound        :
    public virtual mpcl::TBaseException ,
    public mpcl::TWarningException      ,
    private TSqlstateDescriptor
  {

    public:

      /**
      *  Builds a new instance.
      *  @param pkcFILE_NAME  Filename where the exception has occurred.
      *  @param kiLINE_NUMBER Line number in the file where the exception has
      *                       occurred.
      */
      TExceptionUESQL_NotFound ( const char* pkcFILE_NAME  ,
                                 const int   kiLINE_NUMBER );


    public:

      //
      //  S E L E C T O R S
      //

      const std::string what (void) const;

  };  // class TExceptionUESQL_NotFound


  ///  Exception: SQL exception class for SQLERROR condition.
  class TExceptionUESQL_Sqlerror        :
    public virtual mpcl::TBaseException ,
    public mpcl::TErrorException        ,
    private TSqlstateDescriptor
  {

    public:

      /**
      *  Builds a new instance.
      *  @param pkcFILE_NAME  Filename where the exception has occurred.
      *  @param kiLINE_NUMBER Line number in the file where the exception has
      *                       occurred.
      */
      TExceptionUESQL_Sqlerror ( const char* pkcFILE_NAME  ,
                                 const int   kiLINE_NUMBER );


    public:

      //
      //  S E L E C T O R S
      //

      const std::string what (void) const;

  };  // class TExceptionUESQL_Sqlerror


  //
  //  E X P O R T E D   F U N C T I O N S
  //

  //
  //  Simple n-adic functions for embedded function calls.
  //

  /**
  *  Composes a string as the  concatenation of \a rkyHOST_NAME, a solidus, and
  *  then \a rkySCHEMA_NAME.
  */
  TString ComposeSchemaLocation ( const std::string& rkyHOST_NAME   ,
                                     const std::string& rkySCHEMA_NAME );

  /**
  *  Composes a string as the  concatenation of \a rkyUSER_NAME, a solidus, and
  *  then \a rkyUSER_PASSWORD.
  */
  TString ComposeUserLogin ( const std::string& rkyUSER_NAME     ,
                                const std::string& rkyUSER_PASSWORD );

  /**
  *  Converts \a gVALUE to a string.
  *  This function is used in terminal <embedded n-adic function call>.
  *  @param gVALUE Source boolean value.
  *  @return A string with "0" (if value is false) or "1" (if value is true).
  */
  TString ToString (bool gVALUE);
  
  /**
  *  Converts \a ldNUMBER to a string.
  *  This function is used in terminal <embedded n-adic function call>.
  *  @param ldNUMBER Source number.
  *  @return A string with the value of \a ldNUMBER.
  */
  TString ToString (long double ldNUMBER);
  
  /**
  *  Converts \a liNUMBER to a string.
  *  This function is used in terminal <embedded n-adic function call>.
  *  @param liNUMBER Source number.
  *  @return A string with the value of \a liNUMBER.
  */
  TString ToString (long int liNUMBER);
  
  /**
  *  Converts \a luiNUMBER to a string.
  *  This function is used in terminal <embedded n-adic function call>.
  *  @param luiNUMBER Source number.
  *  @return A string with the value of \a luiNUMBER.
  */
  TString ToString (long unsigned int luiNUMBER);
  
}  // namespace uesql


#endif  // not _UESQLC_DEFS_UESQLC__
