
#ifndef _POSTGRESQL_LIBPQ_6_2_X__
#define _POSTGRESQL_LIBPQ_6_2_X__

#include <cstdlib>
#include <libpq-fe.h>
#include <mpcl/smart_pointer.hh>
#include <mpcl/string.hh>
#include <uesqlc/base_connection_manager.hh>
#include <uesqlc/defs.hh>


///  Name-space for UESQL.
namespace uesql
{

  namespace postgresql_6_2_x
  {

    ///  Type with connection with host, database and handle triple.
    struct TConnectionTriple
    {

      public:

	PGconn*         ptPgconn;
	mpcl::TString   yDatabaseName;
	mpcl::TString   yHostName;
        mpcl::TString   yUserName;
        mpcl::TString   yUserPassword;


      public:

	//
	//  C O N S T R U C T O R S
	//

	TConnectionTriple (void) :
	  ptPgconn      ()       ,
	  yDatabaseName ()       ,
	  yHostName     ()       ,
          yUserName     ()       ,
          yUserPassword ()       {}

	TConnectionTriple ( const mpcl::TString& rkyHOST_NAME     ,
			    const mpcl::TString& rkyDATABASE_NAME ,
			    const mpcl::TString& rkyUSER_NAME     ,
			    const mpcl::TString& rkyUSER_PASSWORD ,
			    PGconn*                  ptPGCONN         ) :
	  ptPgconn      (ptPGCONN)                                      ,
	  yDatabaseName (rkyDATABASE_NAME)                              ,
	  yHostName     (rkyHOST_NAME)                                  ,
          yUserName     (rkyUSER_NAME)                                  ,
          yUserPassword (rkyUSER_PASSWORD)                              {}

        void close (void)
        {
          if ( ptPgconn )
          {
            PQfinish (ptPgconn);
            ptPgconn = NULL;
          }
        }

    };  // struct TConnectionTriple
   
  }  // namespace postgresql_6_2_x

  ///  PostgreSQL LIBPQ 6.2.x manager class.
  class TPostgresql62xManager : 
    public TBaseConnectionManager<postgresql_6_2_x::TConnectionTriple, PGresult*>
  {

    protected:

      long int   liTransactionCount;

  
    public:
  
      //
      //  C O N S T R U C T O R S
      //

      ///  Builds a new instance.
      TPostgresql62xManager (void)                                                     :
        TBaseConnectionManager<postgresql_6_2_x::TConnectionTriple, PGresult*>() {}

      ///  Initiates a new transaction.
      void begin (void)
      {
        if ( liTransactionCount++ == 0 )
        {
          executeClean ("BEGIN;");
        }
      }

      ///  Connects to current database and host.
      void connect (const mpcl::TString& rkyCONNECTION_NAME)
      {
        connect ( rkyCONNECTION_NAME                   ,
                  getCurrentConnection().yHostName     ,
                  getCurrentConnection().yDatabaseName ,
                  getCurrentConnection().yUserName     ,
                  getCurrentConnection().yUserPassword );
      }

      ///  Connects to current host.
      void connect ( const mpcl::TString& rkyCONNECTION_NAME ,
                     const mpcl::TString& rkyDATABASE_NAME   )
      {
        connect ( rkyCONNECTION_NAME                   ,
                  getCurrentConnection().yHostName     ,
                  rkyDATABASE_NAME                     ,
                  getCurrentConnection().yUserName     ,
                  getCurrentConnection().yUserPassword );
      }

      void connect ( const mpcl::TString& rkyCONNECTION_NAME ,
                     const mpcl::TString& rkyHOST_NAME       ,
                     const mpcl::TString& rkyDATABASE_NAME   ,
                     const mpcl::TString& rkyUSER_LOGIN      )
      {
        using mpcl::TString;

        TString              yUserName;
        TString              yUserPassword;
        TString::size_type   zBarIndex;

        zBarIndex = rkyUSER_LOGIN.find ('/');
        if ( zBarIndex == TString::npos )
        {
          yUserName = rkyUSER_LOGIN;
          yUserName.suppress ('\'');
        }
        else
        {
          yUserName     = rkyUSER_LOGIN.substr (0, zBarIndex);
          yUserPassword = rkyUSER_LOGIN.substr (zBarIndex + 1);
          yUserName.suppress ('\'');
          yUserPassword.suppress ('\'');
        }
        connect ( rkyCONNECTION_NAME ,
                  rkyHOST_NAME       ,
                  rkyDATABASE_NAME   ,
                  yUserName          ,
                  yUserPassword      );
      }

      void connect ( const mpcl::TString& rkyCONNECTION_NAME ,
                     const mpcl::TString& rkyHOST_NAME       ,
                     const mpcl::TString& rkyDATABASE_NAME   ,
                     const mpcl::TString& rkyUSER_NAME       ,
                     const mpcl::TString& rkyUSER_PASSWORD   );

      /**
      *  Frees statement execution result \a ptPGRESULT.
      *  @param ptPGRESULT Source result.
      */
      void clearResult (PGresult* ptPGRESULT)
      {
        if ( ptPGRESULT )
        {
          PQclear (ptPGRESULT);
        }
      }

      /**
      *  Diagnoses statement result (\a ptPGRESULT).
      *  It updates properly SQL status variables.
      *  @param ptPGRESULT Source statement execution result.
      */
      void diagnose (PGresult* ptPGRESULT);

      /**
      *  Diagnoses connection sanity (first stage diagnostics). It
      *  updates properly SQL status variables.
      *  @param rtCONNECTION_TRIPLE Connection to check.
      */
      void diagnoseConnection (postgresql_6_2_x::TConnectionTriple& rtCONNECTION_TRIPLE);

      void diagnoseSingleRow (PGresult* ptPGRESULT);

// <DELETE>
//       void disconnect (const mpcl::TString kyCONNECTION_NAME);
// </DELETE>

      ///  Ends current transaction.
      void end (void)
      {
        if ( liTransactionCount-- == 1 )
        {
          executeClean ("END;");
        }
        if ( liTransactionCount < 0 )
        {
          throw mpcl::TExceptionEL_Constraint ("unbalanced begin/end", __FILE__, __LINE__);
        }
      }

      PGresult* execute (const char* pkcSTATEMENT);

      PGresult* execute (const mpcl::TString& rkySTATEMENT);

      PGresult* execute (const char* pkcSTATEMENT, const char* pkcCONNECTION_NAME);

      void initialize (void);
  
  };  // class TPostgresql62xManager


  //
  //  E X P O R T E D   V A R I A B L E S
  //

  extern TPostgresql62xManager   htPostgresql62xManager;

}  // namespace uesql


#endif  // not _POSTGRESQL_LIBPQ_6_2_X__
