/*
*  Name:         create-iterators.uesql.cc
*  Author:       Francisco Vides Fernandez
*  Contributor:  Rafael Jesus Alcantara Perez
*  Summary:      Database creation of iterators test
*  Date:         $Date: 2002/08/19 12:59:57 $
*  Revision:     $Revision: 1.3 $
*
*  Copyright (C) 2001-2002  Francisco Vides Fernandez <pax@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <iostream>
#include <mpcl/config_processor.hh>
#include <mpcl/general-exceptions.hh>
#include <mpcl/log.hh>
#include <mpcl/string.hh>
#include <mpcl/test.h>

using uesql::TExceptionUESQL_Sqlerror;


//
//  L O C A L   V A R I A B L E S
//

static uesql::TSqlstateDescriptor   _tSqlstateDescriptor;


//
//  L O C A L   F U N C T I O N S
//

/**
*  Construct error message from SQL status.
*  @return SQL status message.
*/
static mpcl::TString _GetErrorMessage (void)
{

  using mpcl::TString;

  return ( TString (_tSqlstateDescriptor.getClassValue())    +
           ": "                                              +
           TString (_tSqlstateDescriptor.getSubclassValue()) +
           " (" + uesql::SQLMESSAGE + ")"                    );

}  // _GetErrorMessage()


static void _CreateTestSchema ( const mpcl::TString& rkyDBMS_SERVER_NAME ,
                                const mpcl::TString& rkyUSER_NAME        ,
                                const mpcl::TString& rkyUSER_PASSWORD    )
{

  using uesql::ComposeSchemaLocation;
  using uesql::ComposeUserLogin;
  using uesql::TString;

  TString   yDbmsSchemaLocation;
  TString   yDbmsTestSchemaLocation;
  TString   yDbmsUserLogin;

  yDbmsSchemaLocation     = ComposeSchemaLocation (rkyDBMS_SERVER_NAME, rkyUSER_NAME);
  yDbmsTestSchemaLocation = ComposeSchemaLocation (rkyDBMS_SERVER_NAME, "TEST_ITERATORS_SCHEMA");
  yDbmsUserLogin          = ComposeUserLogin (rkyUSER_NAME, rkyUSER_PASSWORD);
  TEST_WRITE_INFO ("DBMS-SCHEMA-LOCATION      = %s", yDbmsSchemaLocation.c_str());
  TEST_WRITE_INFO ("DBMS-USER-LOGIN           = %s", yDbmsUserLogin.c_str());
  TEST_WRITE_INFO ("DBMS-TEST-SCHEMA-LOCATION = %s", yDbmsTestSchemaLocation.c_str());
  extern "UESQL::default"
  {
    --
    --  Connects to default schema.
    --
    connect to :yDbmsSchemaLocation user :yDbmsUserLogin;

    --
    --  Creates schema test_iterators_schema.
    --
    create schema test_iterators_schema
      create table instance
      (
        attrib0   date,
        attrib1   integer,
        attrib2   real,
        attrib3   varchar(20)
      );
    --
    --  Connects to newly created schema
    --
    connect to :yDbmsTestSchemaLocation as 'test_connection' user :yDbmsUserLogin;
  }

}  // _CreateTestSchema()


static void _FillSchema (void)
{

  extern "UESQL::default"
  {
    insert into instance values ('20-12-2000', 0, 0.0, 'valor 0');
    insert into instance values ('20-12-2001', 1, 1.1, 'valor 1');
    insert into instance values ('20-12-2002', 2, 2.2, 'valor 2');
  }

}   // _FillSchema()


int main (int argc, const char** argv)
{

  mpcl::TConfigProcessor   tConfigProcessor (argc, argv);

  TEST_INIT ("Database creation for iterators test");
  TEST_START_WATCH;
  try
  {
    tConfigProcessor.addOption ("dbms-server-name",   "o", "dbms");
    tConfigProcessor.addOption ("dbms-user-name",     "u", "test");
    tConfigProcessor.addOption ("dbms-user-password", "p", "test");
    tConfigProcessor["dbms-server-name"].setUsageString ("Server where database resides");
    tConfigProcessor["dbms-user-name"].setUsageString ("");
    tConfigProcessor["dbms-user-password"].setUsageString ("");
    tConfigProcessor.processOptions();
    
    if ( !TEST_IN_SILENT_MODE )
    {
      mpcl::tLog.also (std::cerr);
    }
    extern "UESQL::default" 
    {
      --
      --  Whenever any SQLERROR condition throw an exception
      --
      whenever sqlerror throw TExceptionUESQL_Sqlerror;
      
    }
    _CreateTestSchema ( tConfigProcessor ["dbms-server-name"].getValue()   ,
                        tConfigProcessor ["dbms-user-name"].getValue()     ,
                        tConfigProcessor ["dbms-user-password"].getValue() );
    _FillSchema();
  }
  catch (const uesql::TExceptionUESQL_Sqlerror& rktEXCEPTION)
  {
    std::cerr << "SQLSTATE="    << uesql::SQLSTATE    << std::endl;
    std::cerr << "SQLCODE="     << uesql::SQLCODE     << std::endl;
    std::cerr << "DESCRIPTION=" << _GetErrorMessage() << std::endl;
    TEST_FAIL_WITH_CODE (uesql::SQLCODE);
  }
  catch (const mpcl::TBaseException& rktEXCEPTION)
  {
    std::cerr << rktEXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }  
  catch (...)
  {
    std::cerr << "Unhandled exception" << std::endl;
    TEST_FAIL_WITH_CODE (3);
  }
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::tLog.erase (std::cerr);
  }
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
